# -*- coding: utf-8 -*-
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer retrain main processing

   Functions to retrain a modeling resource


"""
from __future__ import absolute_import

import sys
import os
import shutil

import bigmler.processing.args as a
import bigmler.utils as u


from bigmler.defaults import DEFAULTS_FILE
from bigmler.retrain.retrain import retrain_model
from bigmler.dispatcher import (SESSIONS_LOG,
                                clear_log_files)
from bigmler.command import get_context


COMMAND_LOG = u".bigmler_retrain"
DIRS_LOG = u".bigmler_retrain_dir_stack"
LOG_FILES = [COMMAND_LOG, DIRS_LOG, u.NEW_DIRS_LOG]

DEFAULT_OUTPUT = "retrain_script"

SETTINGS = {
    "command_log": COMMAND_LOG,
    "sessions_log": SESSIONS_LOG,
    "dirs_log": DIRS_LOG,
    "default_output": DEFAULT_OUTPUT,
    "defaults_file": DEFAULTS_FILE}


def check_compulsory_options(flags, args):
    """Checks whether the id or a unique tag are provided

    """
    return args.resource_id is not None or \
        len([flag for flag in flags if flag.endswith("-tag")]) > 0


def retrain_dispatcher(args=sys.argv[1:]):
    """Main processing of the parsed options for BigMLer retrain

    """

    # If --clear-logs the log files are cleared
    if "--clear-logs" in args:
        clear_log_files(LOG_FILES)

    # parses the command line to get the context args and the log files to use
    command_args, command, api, session_file, resume = get_context(args,
                                                                   SETTINGS)

    # --id or --model-tag, --ensemble-tag, etc. is compulsory
    if check_compulsory_options(command.flags, command_args):
        retrain_model(command_args, api, command.common_options,
                      session_file=session_file)
        u.log_message("_" * 80 + "\n", log_file=session_file)
    else:
        sys.exit("You must provide the ID of the resource to be"
                 " retrained in the --id option or a unique tag"
                 " to retrieve such ID."
                 " Type bigmler retrain --help\n"
                 " to see all the available options.")



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2016-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing topic model predictions creation

"""
from __future__ import absolute_import

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module,
                                 teardown_class)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_batch_tst_prediction_steps as batch_pred
import bigmler.tests.basic_topic_model_steps as topic_pred

def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestTopicModel()
    test.setup_scenario02()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestTopicModel(object):

    def setup(self):
        """
            Debug information
        """
        print "\n-------------------\nTests in: %s\n" % __name__

    def teardown(self):
        """Calling generic teardown for every method

        """
        self.world = teardown_class()
        print "\nEnd of tests in: %s\n-------------------\n" % __name__

    def test_scenario01(self):
        """
        Scenario: Successfully building topic model test distributions from dataset:
            Given I created the dataset in setup_scenario02
            And I create BigML topic model resources from dataset to test "<test>" with options "<options>" and log predictions in "<output>"
            And I check that the topic model has been created
            And I check that the topic distributions are ready
            Then the local topic distribution file is like "<topic_distribution_file>"

            Examples:
            | test                    | options                  | output                                   |topic_distribution_file           |
            | ../data/spam.csv        | --test-separator="\t --prediction-header"     |./scenario1_td/topic_distributions.csv   | ./check_files/topic_distributions_spam.csv   |


        """
        print self.test_scenario01.__doc__
        examples = [
            ['data/spam.csv', '--test-separator="\t" --prediction-header',
             'scenario1_td/topic_distributions.csv',
             'check_files/topic_distributions_spam.csv',
             './check_files/topic_distributions_spam.csv']]
        for example in examples:
            print "\nTesting with:\n", example
            topic_pred.i_create_all_td_resources_from_dataset( \
                self, example[0], example[1], example[2])
            topic_pred.i_check_create_topic_model(self)
            topic_pred.i_check_create_topic_distributions(self)
            topic_pred.i_check_topic_distributions(self, example[3])


    def setup_scenario02(self):
        """
        Scenario: Successfully building text source from local file:
            Given I create BigML dataset uploading train "<data>" file with attributes "<attributes>" in "<output>"
            And I check that the source has been created
            Then I check that the dataset has been created

            Examples:
            | data               | attributes                     | output                 |
            | ../data/spam.csv   | ../data/spam_attributes.json   |  scenario2_td/topic_distributions.csv  |
        """
        print self.setup_scenario02.__doc__
        examples = [
            ['data/spam.csv', 'data/spam_attributes.json', 'scenario2_td/topic_distributions.csv']]
        for example in examples:
            print "\nTesting with:\n", example
            test_pred.i_create_dataset_with_attributes( \
                self, data=example[0], attributes=example[1], output=example[2])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)



    def test_scenario03(self):
        """
        Scenario: Successfully building test predictions from source
            Given I created the dataset in setup_scenario02
            And I create BigML topic model resources from source to test "<test>" with options "<options>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the topic model has been created
            And I check that the topic distributions are ready
            Then the local topic distribution file is like "<topic_distribution_file>"

            Examples:
            | test                    | options                  | output                                   |topic_distribution_file           |
            | ../data/spam.csv        | --test-separator="\t --prediction-header"     |./scenario3_td/topic_distributions.csv   | ./check_files/topic_distributions_spam.csv   |


        """
        print self.test_scenario03.__doc__
        examples = [
            ['data/spam.csv', '--test-separator="\t" --prediction-header',
             'scenario3_td/topic_distributions.csv',
             'check_files/topic_distributions_spam.csv',
             './check_files/topic_distributions_spam.csv']]
        for example in examples:
            print "\nTesting with:\n", example
            topic_pred.i_create_all_td_resources_from_source( \
                self, example[0], example[1], example[2])
            test_pred.i_check_create_dataset(self, suffix=None)
            topic_pred.i_check_create_topic_model(self)
            topic_pred.i_check_create_topic_distributions(self)
            topic_pred.i_check_topic_distributions(self, example[3])


    def test_scenario04(self):
        """
        Scenario: Successfully building test predictions from topic model
            Given I created the dataset in setup_scenario02
            And I create topic model from dataset
            And I create BigML topic model resources from model to test "<test>" with options "<options>" and log predictions in "<output>"
            And I check that the dataset has been created
            And I check that the topic distributions are ready
            Then the local topic distribution file is like "<topic_distribution_file>"

            Examples:
            | test                    | options                  | output                                   |topic_distribution_file           |
            | ../data/spam.csv        | --test-separator="\t --prediction-header"     |./scenario4_td/topic_distributions.csv   | ./check_files/topic_distributions_spam.csv   |


        """
        print self.test_scenario04.__doc__
        examples = [
            ['data/spam.csv', '--test-separator="\t" --prediction-header',
             'scenario4_td/topic_distributions.csv',
             'check_files/topic_distributions_spam.csv',
             './check_files/topic_distributions_spam.csv']]
        for example in examples:
            print "\nTesting with:\n", example
            topic_pred.i_create_topic_model_from_dataset(self, example[2])
            topic_pred.i_check_create_topic_model(self)
            topic_pred.i_create_all_td_resources_from_model( \
                self, example[0], example[1], example[2])
            topic_pred.i_check_create_topic_distributions(self)
            topic_pred.i_check_topic_distributions(self, example[3])


    def test_scenario05(self):
        """
        Scenario: Successfully building batch topic distributions from topic model
            Given I created the dataset in setup_scenario02
            And I create BigML batch topic distribution resources from dataset to test "<test>" and log predictions in "<output>"
            And I check that the topic model has been created
            And I check that the batch topic distribution is ready
            Then the local topic distribution file is like "<predictions_file>"

            Examples:
            | test                    | options                  | output                                   |topic_distribution_file           |
            | ../data/spam.csv        | --test-separator="\t --prediction-header" | ./scenario5_td/topic_distributions.csv   | ./check_files/topic_distributions_spam.csv   |


        """
        print self.test_scenario05.__doc__
        examples = [
            ['data/spam.csv', '--test-separator="\t" --prediction-header',
             'scenario5_td/topic_distributions.csv',
             'check_files/topic_distributions_spam.csv',
             './check_files/topic_distributions_spam.csv']]
        for example in examples:
            print "\nTesting with:\n", example
            topic_pred.i_create_topic_model_from_dataset(self, example[2])
            topic_pred.i_check_create_topic_model(self)
            topic_pred.i_create_topic_distribution_from_model_remote(self, test=example[0], options=example[1], output=example[2])
            topic_pred.i_check_create_topic_distributions(self)
            # the check between local and remote predictions is removed till we
            # add new options in the local side to match the remote ones
            # topic_pred.i_check_topic_distributions(self, example[3])



# -*- coding: utf-8 -*-
#
# Auxiliar class: Simple stoppable HTTPServer extracted from
#
# http://code.activestate.com/recipes/425210-simple-stoppable-server-using-socket-timeout/
#
# Unless required by applicable law or agreed to in writing, this software
# is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

import BaseHTTPServer
import socket

class StoppableHTTPServer(BaseHTTPServer.HTTPServer):

    def server_bind(self):
        BaseHTTPServer.HTTPServer.server_bind(self)
        self.socket.settimeout(1)
        self.run = True

    def get_request(self):
        while self.run:
            try:
                sock, addr = self.socket.accept()
                sock.settimeout(None)
                return (sock, addr)
            except socket.timeout:
                pass

    def stop(self):
        self.run = False

    def serve(self):
        while self.run:
            self.handle_request()



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Tree level output for MySQL

This module defines functions that generate MySQL SQL code to make local
predictions
"""
import sys

from bigml.tree_utils import slugify, INDENT, sort_fields, docstring_comment, \
    MAX_ARGS_LENGTH, TERM_OPTIONS
from bigml.model import Model

from bigmler.export.out_tree.mysqltree import MySQLTree

class MySQLModel(Model):


    def __init__(self, model, api=None, fields=None):
        """Empty attributes to be overriden

        """
        self.tree_class = MySQLTree
        Model.__init__(self, model, api, fields)

    def plug_in(self, out=sys.stdout, filter_id=None, subtree=True, attr=None):
        """Generates a basic MySQL SQL expression that implements a function
           describing the model.

        `out`  is file descriptor to write the MySQL code.
        `attr` is used to predict an attribute (e.g 'confidence') other
               than the output

        """
        ids_path = self.get_ids_path(filter_id)
        length = self.mysql(out, ids_path=ids_path, subtree=subtree,
                            attr=attr)
        if length > 0:
            out.write(u"\n\n")
        else:
            sys.exit(u"\nFailed to represent this model "
                     u"in MySQL syntax. Currently only models with "
                     u"categorical and numeric fields can be generated.\n")
        out.flush()

    def mysql(self, out, ids_path=None, subtree=True, attr=None):
        """Writes a MySQL function that implements the model.

        """

        definition = "CREATE FUNCTION predict_%s (%s)" \
                     "\nRETURNS %s DETERMINISTIC\nRETURN "
        args = []
        parameters = sort_fields(self.fields)
        for field_id, field in parameters:
            field_name_to_show = self.fields[field_id]['name'].strip()
            field_type = 'NUMERIC' if field['optype'] == 'numeric' else \
                'VARCHAR(250)'
            if field_name_to_show == "":
                field_name_to_show = field_id
            if field_id != self.objective_id:
                args.append("`%s` %s" % (field_name_to_show,
                                         field_type))
        objective = self.fields[self.objective_id]
        function_name = objective['name']
        return_type = 'NUMERIC' if objective['optype'] == 'numeric' else \
            'VARCHAR(250)'
        if function_name == "":
            function_name = "field_" + self.objective_id
        # when the output is a confidence metric (error/confidence)
        if attr is not None:
            function_name += "_%s" % attr
            return_type = 'NUMERIC'
        definition = definition % (function_name, ", ".join(args), return_type)
        out.write(definition)
        body = self.tree.plug_in_body(ids_path=ids_path, subtree=subtree,
                                      attr=attr)

        out.write(body)
        out.flush()
        return len(body)



# -*- coding: utf-8 -*-
#
# Copyright 2014-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import absolute_import


import os
import time
import csv
import json

from nose.tools import assert_equal, assert_not_equal, ok_
from bigmler.tests.world import world, res_filename
from subprocess import check_call, CalledProcessError
from bigml.api import check_resource
from bigmler.utils import storage_file_name
from bigmler.checkpoint import file_number_of_lines
from bigmler.tests.common_steps import check_debug


def shell_execute(command, output, test=None, options=None,
                  data=None, test_split=None):
    """Excute bigmler command in shell

    """
    command = check_debug(command)
    world.directory = os.path.dirname(output)
    world.folders.append(world.directory)
    try:
        retcode = check_call(command, shell=True)
        if retcode < 0:
            assert False
        else:
            if test is not None:
                world.test_lines = file_number_of_lines(test)
                world.test_lines -= 1
                # prediction file has headers in it,
                # so first line must be ignored
                world.prediction_header = options is not None and \
                    options.find('--prediction-header') > -1
            if test_split is not None:
                data_lines = file_number_of_lines(data) - 1
                world.test_lines = int(data_lines * float(test_split))
            world.output = output
            assert True
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)


#@step(r'I create BigML resources uploading train "(.*?)" file to create centroids for "(.*?)" and log predictions in "([^"]*)"$')
def i_create_all_cluster_resources(step, data=None, test=None, output=None):
    ok_(data is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler cluster --train " + res_filename(data) + " --test " + test +
               " --k 8" +
               " --store --output " + output)
    shell_execute(command, output, test=test)


#@step(r'I check that the cluster has been created')
def i_check_create_cluster(step):
    cluster_file = "%s%sclusters" % (world.directory, os.sep)
    try:
        cluster_file = open(cluster_file, "r")
        cluster = check_resource(cluster_file.readline().strip(),
                                 world.api.get_cluster)
        world.clusters.append(cluster['resource'])
        world.cluster = cluster
        cluster_file.close()
        assert True
    except Exception, exc:
        assert False, str(exc)


#@step(r'I check that the centroids are ready')
def i_check_create_centroids(step):
    predictions_file = world.output
    predictions_file = open(predictions_file, "r")
    predictions_lines = 0
    for line in predictions_file:
        predictions_lines += 1
    predictions_file.close()
    if world.prediction_header:
        predictions_lines -= 1
    assert_equal(predictions_lines, world.test_lines,
                 msg="predictions lines: %s, test lines: %s" % \
                 (predictions_lines, world.test_lines))


#@step(r'the local centroids file is like "(.*)"')
def i_check_centroids(step, check_file):
    check_file = res_filename(check_file)
    predictions_file = world.output
    try:
        predictions_file = csv.reader(open(predictions_file, "U"), lineterminator="\n")
        check_file = csv.reader(open(check_file, "U"), lineterminator="\n")
        for row in predictions_file:
            check_row = check_file.next()
            if len(check_row) != len(row):
                assert False
            for index in range(len(row)):
                if check_row[index] != row[index]:
                    print row, check_row
                    assert False
        assert True
    except Exception, exc:
        assert False, str(exc)


#@step(r'I create BigML resources using dataset to find centroids for "(.*)" and log predictions in "(.*)"')
def i_create_cluster_resources_from_dataset(step, test=None, output=None):
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler cluster --dataset " +
               world.dataset['resource'] + " --test " + test +  " --k 8" +
               " --store --output " + output)
    shell_execute(command, output, test=test)


#@step(r'I create BigML resources using source to find centroids for "(.*)" and log predictions in "(.*)"')
def i_create_cluster_resources_from_source(step, test=None, output=None):
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler cluster --source " +
               world.source['resource'] + " --test " + test + " --k 8" +
               " --store --output " + output)
    shell_execute(command, output, test=test)


#@step(r'I create BigML resources using local cluster in "(.*)" to find centroids for "(.*)" and log predictions in "(.*)"')
def i_create_cluster_resources_from_local_cluster(step, directory=None, test=None, output=None):
    ok_(test is not None and output is not None and directory is not None)
    test = res_filename(test)
    with open(os.path.join(directory, "clusters")) as handler:
        cluster_id = handler.readline().strip()
    command = ("bigmler cluster --cluster-file " +
               storage_file_name(directory, cluster_id) +
               " --test " + test +
               " --store --output " + output)
    shell_execute(command, output, test=test)


#@step(r'I create BigML resources using cluster to find centroids for "(.*)" and log predictions in "(.*)"')
def i_create_cluster_resources_from_cluster(step, test=None, output=None):
    ok_(test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler cluster --cluster " +
               world.cluster['resource'] + " --test " + test + " --k 8" +
               " --store --output " + output)
    shell_execute(command, output, test=test)


#@step(r'I create BigML resources using clusters in file "(.*)" to find centroids for "(.*)" and log predictions in "(.*)"')
def i_create_cluster_resources_from_clusters_file(step, clusters_file=None, test=None, output=None):
    ok_(test is not None and output is not None and clusters_file is not None)
    test = res_filename(test)
    command = ("bigmler cluster --clusters " +
               clusters_file + " --test " + test +
               " --store --output " + output)
    shell_execute(command, output, test=test)


#@step(r'I create BigML resources uploading train "(.*?)" file to find centroids for "(.*?)" remotely to dataset with no CSV and log resources in "([^"]*)"$')
def i_create_all_cluster_resources_to_dataset(step, data=None, test=None, output_dir=None):
    ok_(data is not None and test is not None and output_dir is not None)
    test = res_filename(test)
    command = ("bigmler cluster --remote --train " + res_filename(data) +
               " --test " + test + " --k 8" +
               " --to-dataset --no-csv " +
               " --store --output-dir " + output_dir)
    shell_execute(command, "%s/x.csv" % output_dir, test=test)


#@step(r'I create BigML resources uploading train "(.*?)" file to find centroids for "(.*?)" remotely with mapping file "(.*)" and log predictions in "([^"]*)"$')
def i_create_all_cluster_resources_with_mapping(step, data=None, test=None, fields_map=None, output=None):
    ok_(data is not None and test is not None and output is not None and fields_map is not None)
    test = res_filename(test)
    command = ("bigmler cluster --remote --train " + res_filename(data) +
               " --test " + test + " --k 8" +
               " --fields-map " + res_filename(fields_map) +
               " --store --output " + output)
    shell_execute(command, output, test=test)


#@step(r'I create BigML resources uploading train "(.*?)" file to find centroids for "(.*?)" remotely with predictions fields "(.*)" and log predictions in "([^"]*)"$')
def i_create_all_cluster_resources_with_prediction_fields(step, data=None, test=None, prediction_fields=None, output=None):
    ok_(data is not None and test is not None and output is not None and prediction_fields is not None)
    test = res_filename(test)
    command = ("bigmler cluster --remote --train " + res_filename(data) +
               " --test " + test + " --k 8" +
               " --prediction-fields \"" + prediction_fields +
               "\" --prediction-info full --prediction-header --store " +
               "--output " + output)
    shell_execute(command, output, test=test, options='--prediction-header')



#@step(r'I create BigML cluster using dataset and summary_fields "(.*)" and log resources in "(.*)"')
def i_create_cluster_from_dataset_with_summary_fields(step, summary_fields=None, output_dir=None):
    ok_(summary_fields is not None and output_dir is not None)
    command = ("bigmler cluster --dataset " +
               world.dataset['resource'] + " --summary-fields " +
               summary_fields + " --k 8" +
               " --store --output-dir " + output_dir)
    shell_execute(command, "%s/x.csv" % output_dir)

#@step(r'I generate datasets for "(.*?)" centroids and log predictions in "(.*?)"$')
def i_create_datasets_from_cluster(step, centroids=None, output=None):
    ok_(centroids is not None and output is not None)
    command = ("bigmler cluster --cluster " + world.cluster['resource'] +
               " --cluster-datasets \"" + centroids +
               "\" --store --output " + output)
    shell_execute(command, output, test=None)


#@step(r'I check that the (\d+) cluster datasets are ready$')
def i_check_cluster_datasets(step, datasets_number=None):

    try:
        datasets_file = os.path.join(world.directory, "dataset_cluster")
        datasets_file = open(datasets_file, "r")
        dataset_ids = datasets_file.readlines()
        world.datasets.extend(dataset_ids)
        if int(datasets_number) == len(dataset_ids):
            assert True
        else:
            assert False, "generated datasets %s, expected %s" % (
                len(dataset_ids), datasets_number)
    except Exception, exc:
        assert False, str(exc)


#@step(r'I check that the (\d+) cluster models are ready$')
def i_check_cluster_models(step, models_number=None):

    try:
        models_file = os.path.join(world.directory, "models_cluster")
        models_file = open(models_file, "r")
        model_ids = models_file.readlines()
        world.models.extend(model_ids)
        if int(models_number) == len(model_ids):
            assert True
        else:
            assert False, "generated models %s, expected %s" % (
                len(model_ids), models_number)
    except Exception, exc:
        assert False, str(exc)


#@step(r'I generate models for "(.*?)" centroids and log results in "(.*?)"$')
def i_create_models_from_cluster(step, centroids=None, output=None):
    ok_(centroids is not None and output is not None)
    command = ("bigmler cluster --dataset " + world.dataset['resource'] +
               " --cluster-models \"" + centroids +
               "\" --k 4 --store --output " + output)
    shell_execute(command, output, test=None)


#@step(r'I check that the cluster has summary fields "(.*)"$')
def i_check_cluster_has_summary_fields(step, summary_fields=None):

    assert_equal(world.cluster['object']['summary_fields'],
                 summary_fields.split(","))



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Tree level output for MySQL function

This module defines functions that generate MySQL SQL code to make local
predictions
"""

from bigml.tree_utils import (filter_nodes, split, ruby_string,
                              missing_branch, none_value,
                              one_branch, INDENT)

from bigml.tree import Tree

T_MISSING_OPERATOR = {
    "=": "ISNULL(",
    "!=": "NOT ISNULL("
}

# Map operator str to its corresponding mysql operator
MYSQL_OPERATOR = {
    "/=": "!="}

def value_to_print(value, optype):
    """String of code that represents a value according to its type

    """
    if (value is None):
        return "NULL"
    if (optype == 'numeric'):
        return value
    return u"'%s'" % value.replace("'", '\\\'')


class MySQLTree(Tree):


    def missing_check_code(self, field, alternate, cmv, attr=None):
        """Builds the code to predict when the field is missing

        """


        condition = "ISNULL(`%s`)" % self.fields[field]['name']
        code = (u"%s (%s)" %
                 (alternate, condition))

        # used when printing the confidence metric
        if attr != None:
            value = getattr(self, attr)
        else:
            value = value_to_print( \
                self.output,
                self.fields[self.objective_id]['optype'])

        code += (u", %s" % value)
        cmv.append(self.fields[field]['name'])

        return code

    def missing_prefix_code(self, field, cmv):
        """Part of the condition that checks for missings when missing_splits
        has been used

        """

        negation = u"" if child.predicate.missing else u"NOT "
        connection = u"OR" if child.predicate.missing else u"AND"
        if not child.predicate.missing:
            cmv.append(self.fields[field]['name'])
        return u"(%sISNULL(`%s`) %s " % ( \
            negation, self.fields[field]['name'],
            connection)

    def split_condition_code(self, field, alternate,
                             pre_condition):
        """Condition code for the split

        """

        post_condition = u""
        optype = self.fields[field]['optype']
        value = value_to_print(self.predicate.value, optype)
        operator = ("" if self.predicate.value is None else
                    MYSQL_OPERATOR.get(self.predicate.operator,
                                       self.predicate.operator))
        if self.predicate.value is None:
            value = u""
            pre_condition = (
                T_MISSING_OPERATOR[self.predicate.operator])
            post_condition = ")"

        condition = u"%s`%s`%s%s%s" % ( \
            pre_condition,
            self.fields[self.predicate.field]['name'],
            operator,
            value,
            post_condition)
        return u"%s (%s)" % (alternate, condition)

    def plug_in_body(self, depth=0, cmv=None,
                     ids_path=None, subtree=True, body=u"", attr=None):
        """Translate the model into a mysql function

        `depth` controls the size of indentation. As soon as a value is missing
        that node is returned without further evaluation.
        `attr` is used to decide the value returned by the function. When
        it's set to None, the prediction is returned. When set to the
        name of an attribute (e.g. 'confidence') this attribute is returned

        """

        if cmv is None:
            cmv = []

        if body:
            alternate = u",\n%sIF (" % (depth * INDENT)
        else:
            alternate = u"IF ("
        post_missing_body = u""


        children = filter_nodes(self.children, ids=ids_path,
                                subtree=subtree)
        if children:

            # field used in the split
            field = split(children)

            has_missing_branch = (missing_branch(children) or
                                  none_value(children))
            # the missing is singled out as a special case only when there's
            # no missing branch in the children list
            if (not has_missing_branch and
                    not self.fields[field]['name'] in cmv):
                body += self.missing_check_code(field, alternate, cmv, attr)
                depth += 1
                alternate = u",\n%sIF (" % (depth * INDENT)
                post_missing_body += u")"

            for child in children:
                pre_condition = u""
                # code when missing splits has been used
                if has_missing_branch and child.predicate.value is not None:
                    pre_condition = self.missing_prefix_code(child, field, cmv)

                # complete split condition code
                body += child.split_condition_code( \
                    field, alternate, pre_condition)

                depth += 1
                alternate = u",\n%sIF (" % (depth * INDENT)
                body = child.plug_in_body(depth, cmv=cmv[:],
                                          ids_path=ids_path, subtree=subtree,
                                          body=body, attr=attr)
            body += u", NULL))" + post_missing_body
            post_missing_body = u""
        else:
            if attr is None:
                value = value_to_print( \
                    self.output, self.fields[self.objective_id]['optype'])
            else:
                value = getattr(self, attr)
            body += u", %s" % (value)

        return body



# -*- coding: utf-8 -*-
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Time-series forecast auxiliary functions

"""
from __future__ import absolute_import


import sys

import bigml.api

from bigml.timeseries import TimeSeries
from bigml.io import UnicodeWriter

import bigmler.utils as u
import bigmler.checkpoint as c

from bigmler.resources import NORMAL_FORMAT, FULL_FORMAT
from bigmler.resources import create_forecast
from bigmler.prediction import use_prediction_headers


def write_forecasts(forecast, output):
    """Writes the final forecast to the required output

    The function creates a new file per field used in the forecast input data.
    The id of the field will be appended to the name provided in the `output`
    parameter.
    """

    for objective_id, forecast_value in forecast.items():
        headers = [f["model"] for f in forecast_value]
        points = []
        if not forecast_value:
            sys.exit("No forecasts available")
        for index in range(len(forecast_value[0]["point_forecast"])):
            points.append([f["point_forecast"][index] for f in forecast_value])
        output_file = "%s_%s.csv" % (output, objective_id)
        with UnicodeWriter(output_file, lineterminator="\n") as out_handler:
            out_handler.writerow(headers)
            for row in points:
                out_handler.writerow(row)


def forecast(time_series, args, session_file=None):
    """Computes a time-series forecast

    """

    local_time_series = TimeSeries(time_series,
                                   api=args.retrieve_api_)

    output = args.predictions
    # Local forecasts: Forecasts are computed locally
    message = u.dated("Creating local forecasts.\n")
    u.log_message(message, log_file=session_file, console=args.verbosity)
    input_data = []
    if args.test_set is not None:
        input_data = [u.read_json(args.test_set)]
    elif args.horizon is not None:
        input_data = [{local_time_series.objective_id: { \
            "horizon": args.horizon}}]
    write_forecasts(local_time_series.forecast(*input_data),
                    output)


def remote_forecast(time_series,
                    forecast_args, args,
                    api, resume, prediction_file=None, session_file=None,
                    path=None, log=None):
    """Computes a remote forecast.

    """

    time_series_id = bigml.api.get_time_series_id( \
        time_series)
    # if resuming, try to extract dataset form log files
    if resume:
        message = u.dated("Forecast not found. Resuming.\n")
        resume, forecast = c.checkpoint(
            c.is_forecast_created, path, debug=args.debug,
            message=message, log_file=session_file, console=args.verbosity)
    if not resume:
        local_time_series = TimeSeries(time_series,
                                       api=args.retrieve_api_)
        output = args.predictions
        if args.test_set is not None:
            input_data = u.read_json(args.test_set)
        elif args.horizon is not None:
            input_data = {local_time_series.objective_id: { \
                "horizon": args.horizon}}

        forecast = create_forecast(
            time_series_id, input_data, forecast_args,
            args, api, session_file=session_file, path=path, log=log)

        write_forecasts(forecast["object"]["forecast"]["result"], output)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing time series forecast creation

"""
from __future__ import absolute_import

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module,
                                 teardown_class)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_time_series_steps as ts_pred

def setup_module():
    """Setup for the module

    """
    common_setup_module()
    test = TestPrediction()
    test.setup_scenario02()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestPrediction(object):

    def setup(self):
        """
            Debug information
        """
        print "\n-------------------\nTests in: %s\n" % __name__

    def teardown(self):
        """Calling generic teardown for every method

        """
        self.world = teardown_class()
        print "\nEnd of tests in: %s\n-------------------\n" % __name__

    def test_scenario01(self):
        """
        Scenario: Successfully building time series test predictions from start with no headers:
            Given I create BigML time series resources uploading train "<data>" file with no headers and log forecasts in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the time series model has been created
            Then the local forecast file is like "<forecast_file>"

            Examples:
            | data                    | output                        |forecasts_file
            | ../data/grades_nh.csv   | ./scenario1_ts_nh/forecast   | ./check_files/forecasts_grades_final.csv


        """
        print self.test_scenario01.__doc__
        examples = [
            ['data/grades_nh.csv', 'scenario1_ts_nh/forecast', 'check_files/forecasts_grades_final.csv']]
        for example in examples:
            print "\nTesting with:\n", example
            ts_pred.i_create_all_ts_resources_with_no_headers(self, example[0], example[1])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            ts_pred.i_check_create_time_series(self)
            ts_pred.i_check_forecasts(self, example[2])

    def setup_scenario02(self):
        """
        Scenario: Successfully building test forecasts from start:
            Given I create BigML time series resources uploading train "<data>" file to test "<test>" and log forecasts in "<output>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the time series has been created
            Then the local forecasts file is like "<forecasts_file>"

            Examples:
            | data               | test                    | output                        |forecasts_file
            | ../data/grades.csv   | ./data/test_grades.json   | ./scenario1_ts/forecasts
        """
        print self.setup_scenario02.__doc__
        examples = [
            ['data/grades.csv', 'data/test_grades.json', 'scenario1_ts/forecasts', 'check_files/forecasts_grades_final.csv', 'scenario1_ts/forecasts_000005.csv']]
        for example in examples:
            print "\nTesting with:\n", example
            ts_pred.i_create_all_ts_resources(self, example[0], example[1], example[2])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            ts_pred.i_check_create_time_series(self)
            ts_pred.i_check_forecasts(self, example[3])



# -*- coding: utf-8 -*-
#
# Copyright 2014-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import absolute_import


import os
import time
import csv
import json

from bigmler.tests.world import world, res_filename
from subprocess import check_call, CalledProcessError
from bigmler.checkpoint import file_number_of_lines
from bigmler.utils import SYSTEM_ENCODING, PYTHON3
from bigml.api import check_resource, BigML
from bigmler.tests.common_steps import check_debug
from nose.tools import assert_equal, assert_not_equal, ok_


#@step(r'I create a BigML source uploading train "(.*)" file and associate it to a new project named "(.*)" storing results in "(.*)"')
def i_create_source_with_project(step, data=None, project=None, output_dir=None):
    ok_(data is not None)
    world.directory = output_dir
    world.folders.append(world.directory)
    #Check if the project already exists
    previous_projects = world.api.list_projects('name=%s' % project)
    while previous_projects['meta']['total_count'] > 0:
        print "the project %s already exists, trying with:" % project
        project += " " + project
        print project
        previous_projects = world.api.list_projects('name=%s' % project)
    try:
        command = (u"bigmler --train " + res_filename(data) +
                   u" --no-model --no-dataset --store --output-dir " +
                   output_dir +
                   u" --project=\"" + project + "\"")
        if not PYTHON3:
            command = command.encode(SYSTEM_ENCODING)
        command = check_debug(command)
        retcode = check_call(command, shell=True)
        if retcode < 0:
            assert False
        else:
            world.output = output_dir
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)


#@step(r'I create a BigML project in an organization named "(.*)" storing results in "(.*)"')
def i_create_project_in_org(step, name=None, output_dir=None, organization=None):
    ok_(name is not None and organization is not None)
    world.directory = output_dir
    world.folders.append(world.directory)
    try:
        command = (u"bigmler project --name \"" + name +
                   "\" --organization " + organization +
                   u" --output-dir " + output_dir)
        if not PYTHON3:
            command = command.encode(SYSTEM_ENCODING)
        command = check_debug(command)
        retcode = check_call(command, shell=True)
        if retcode < 0:
            assert False
        else:
            world.output = output_dir
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)


#@step(r'I create a BigML source uploading train "(.*)" file and associate it to the last created project id storing results in "(.*)"')
def i_create_source_with_org_project(step, data=None, output_dir=None):
    ok_(data is not None)
    world.directory = output_dir
    world.folders.append(world.directory)
    try:
        command = (u"bigmler --train " + res_filename(data) +
                   u" --no-model --no-dataset --store --output-dir " +
                   output_dir +
                   u" --org-project " + world.project["resource"])
        if not PYTHON3:
            command = command.encode(SYSTEM_ENCODING)
        command = check_debug(command)
        retcode = check_call(command, shell=True)
        if retcode < 0:
            assert False
        else:
            world.output = output_dir
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)


#@step(r'I create a BigML source uploading train "(.*)" file and associate it to the last created project id storing results in "(.*)"')
def i_create_source_with_project_id(step, data=None, output_dir=None):
    ok_(data is not None)
    world.directory = output_dir
    world.folders.append(world.directory)
    try:
        command = (u"bigmler --train " + res_filename(data) +
                   u" --no-model --no-dataset --store --output-dir " +
                   output_dir +
                   u" --project-id " + world.project['resource'])
        if not PYTHON3:
            command = command.encode(SYSTEM_ENCODING)
        command = check_debug(command)
        retcode = check_call(command, shell=True)
        if retcode < 0:
            assert False
        else:
            world.output = output_dir
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)


#@step(r'the source is associated to the project')
def check_source_in_project(step):
    assert_equal(world.project['resource'], world.source['object']['project'])


#@step(r'I check that the project has been created$')
def i_check_create_project(step, organization=False):
    project_file = "%s%sproject" % (world.directory, os.sep)
    try:
        project_file = open(project_file, "r")
        project = check_resource(project_file.readline().strip(),
                                 world.api.get_project)
        world.projects.append(project['resource'])
        world.project = project
        project_file.close()
    except Exception, exc:
        assert False, str(exc)
    if organization:
        world.api = BigML(world.USERNAME, world.API_KEY, debug=world.debug,
                          project=world.project["resource"])
        print world.api.connection_info()



#@step(r'the source has no project association')
def check_source_in_no_project(step):
    ok_(world.source['object']['project'] is None)

#@step(r'I create a BigML project "(.*)" and log results in "(.*)"')
def i_create_project(step, project=None, output_dir=None):
    ok_(project is not None)
    world.directory = output_dir
    world.folders.append(world.directory)
    try:
        command = (u"bigmler project --name \"" + project +
                   u"\" --store --output-dir " +
                   output_dir)
        if not PYTHON3:
            command = command.encode(SYSTEM_ENCODING)
        command = check_debug(command)
        retcode = check_call(command, shell=True)
        if retcode < 0:
            assert False
        else:
            world.output = output_dir
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)

#@step(r'I update the project params "(.*)" to "(.*)"')
def i_update_project(step, params=None, values=None):
    ok_(params is not None and values is not None)
    try:
        command = (u"bigmler project --project-id " +
                   world.project['resource'] +
                   u" --store --output-dir " +
                   world.directory)
        for index, param in enumerate(params):
            value = values[index]
            command += u" --%s %s" % (param, value)

        if not PYTHON3:
            command = command.encode(SYSTEM_ENCODING)
        command = check_debug(command)
        retcode = check_call(command, shell=True)
        assert retcode >= 0
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)

#@step(r'the project params "(.*)" are "(.*)"')
def check_params_values(step, params=None, values=None):
    for index, param in enumerate(params):
        value = values[index]
        assert_equal(world.project['object'][param], value)


#@step(r'I check that the project has been updated$')
def i_check_update_project(step):
    project = check_resource(world.project['resource'],
                             world.api.get_project)
    world.project = project



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2016-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing logistic regression predictions creation

"""
from __future__ import absolute_import

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module,
                                 teardown_class)


import bigmler.tests.basic_execute_steps as execute

def setup_module():
    """Setup for the module

    """
    common_setup_module()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestExecute(object):

    def setup(self):
        """
            Debug information
        """
        print "\n-------------------\nTests in: %s\n" % __name__

    def teardown(self):
        """Calling generic teardown for every method

        """
        self.world = teardown_class()
        print "\nEnd of tests in: %s\n-------------------\n" % __name__

    def test_scenario01(self):
        """
        Scenario: Successfully creating an execution from source code:
            Given I create BigML execution resources from source code "<code>" and log results in  "<output_dir>"
            And I check that the script has been created
            And I check that the execution has been created
            And I check that the result is ready
            Then the result file is like "<result_file>"

            Examples:
            | code      | output_dir       | result_file
            | (+ 1 1)   | scenario1_exe    | check_files/results_s1exe.json

        """
        print self.test_scenario01.__doc__
        examples = [
            ['(+ 1 1)', 'scenario1_exe', 'check_files/results_s1exe.json' ]]
        for example in examples:
            print "\nTesting with:\n", example
            execute.i_create_all_execution_resources(self, example[0], example[1])
            execute.i_check_create_script(self)
            execute.i_check_create_execution(self)
            execute.i_check_create_result(self)
            execute.i_check_result_is(self, example[2])


    def test_scenario02(self):
        """
        Scenario: Successfully creating a library from file:
            Given I create BigML library from code in file "<code_file>" and log results in  "<output_dir>"
            Then I check that the library has been created

            Examples:
            | code_file         | output_dir
            | code_lib.whizzml      | scenario2_exe

        """
        print self.test_scenario02.__doc__
        examples = [
            ['data/whizzml_lib/lib/code_lib.whizzml', 'scenario2_exe']]
        for example in examples:
            print "\nTesting with:\n", example
            execute.i_create_all_library_resources(self, example[0], example[1])
            execute.i_check_create_library(self)


    def test_scenario03(self):
        """
        Scenario: Successfully creating an execution with input/outputs from a code file:
            Given I create BigML execution resources from code in file "<code_file>" with inputs "<inputs_dec>", outputs "<outputs_dec>" and inputs "<inputs>" and log results in  "<output_dir>"
            And I check that the script has been created
            And I check that the execution has been created
            And I check that the result is ready
            Then the result file is like "<result_file>"

            Examples:
            | code_file      | output_dir       | inputs_dec           | outputs_dec           | inputs           | result_file
            | code.whizzml   | scenario3_exe    | data/inputs_dec.json | data/outputs_dec.json | data/inputs.json | check_files/results_s3exe.json

        """
        print self.test_scenario03.__doc__
        examples = [
            ['data/whizzml/code.whizzml', 'scenario3_exe', 'data/inputs_dec.json', 'data/outputs_dec.json', 'data/inputs.json', 'check_files/results_s3exe.json']]
        for example in examples:
            print "\nTesting with:\n", example
            execute.i_create_all_execution_with_io_resources(self, example[0], example[1], example[2], example[3], example[4])
            execute.i_check_create_script(self)
            execute.i_check_create_execution(self)
            execute.i_check_create_result(self)
            execute.i_check_result_is(self, example[5])


    def test_scenario04(self):
        """
        Scenario: Successfully creating an whizzml package from a metadata file:
            Given I create a BigML whizzml package from "<package_dir>" and log results in  "<output_dir>"
            Then I check that the script in "<package_dir>" has been created

            Examples:
            | package_dir       | output_dir       |
            | data              | scenario4_pck    |

        """
        print self.test_scenario04.__doc__
        examples = [
            ['data/whizzml', 'scenario4_pck']]
        for example in examples:
            print "\nTesting with:\n", example
            execute.i_create_from_whizzml_package(self, example[0], example[1])
            execute.i_check_create_package_script(self, example[0])


    def test_scenario04(self):
        """
        Scenario: Successfully creating an whizzml package from a metadata file embedding library code:
            Given I create a BigML whizzml package from "<package_dir>", embed any library and log results in  "<output_dir>"
            Then I check that the script in "<package_dir>" has been created

            Examples:
            | package_dir                   | output_dir       |
            | data/whizzml_lib              | scenario5_pck    |

        """
        print self.test_scenario04.__doc__
        examples = [
            ['data/whizzml_lib', 'scenario5_pck']]
        for example in examples:
            print "\nTesting with:\n", example
            execute.i_create_from_whizzml_package_embedding(self, example[0], example[1])
            execute.i_check_create_package_script(self, example[0])



# -*- coding: utf-8 -*-
#
# Copyright 2014-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""Options for BigMLer dataset processing

"""

def get_dataset_options(defaults=None):
    """Dataset-related options

    """

    if defaults is None:
        defaults = {}

    options = {
        # If a BigML dataset is provided, the script won't create a new one
        '--dataset': {
            'action': 'store',
            'dest': 'dataset',
            'default': defaults.get('dataset', None),
            'help': "BigML dataset Id."},

        # The path to a file containing dataset ids.
        '--datasets': {
            'action': 'store',
            'dest': 'datasets',
            'default': defaults.get('datasets', None),
            'help': ("Path to a file containing dataset/ids. Just"
                     " one dataset per line"
                     " (e.g., dataset/50a20697035d0706da0004a4).")},

        # If a BigML json file containing a dataset structure is provided,
        # the script will use it.
        '--dataset-file': {
            'action': 'store',
            'dest': 'dataset_file',
            'default': defaults.get('dataset_file', None),
            'help': "BigML dataset JSON structure file."},

        # Sampling to use when using bagging.
        '--sample-rate': {
            'action': 'store',
            'dest': 'sample_rate',
            'default': defaults.get('sample_rate', 1.0),
            'type': float,
            'help': "Sample rate to split datasets."},

        # Fields to include in the dataset.
        '--dataset-fields': {
            'action': 'store',
            'dest': 'dataset_fields',
            'default': defaults.get('dataset_fields', None),
            'help': ("Comma-separated list of field column numbers"
                     " to include in the dataset.")},

        # Path to a file that includes a JSON filter.
        '--json-filter': {
            'action': 'store',
            'dest': 'json_filter',
            'default': defaults.get('json_filter', None),
            'help': "File including a JSON filter."},

        # Path to a file that includes a lisp filter.
        '--lisp-filter': {
            'action': 'store',
            'dest': 'lisp_filter',
            'default': defaults.get('lisp_filter', None),
            'help': "File including a Lisp filter."},

        # Make dataset public.
        '--public-dataset': {
            'action': 'store_true',
            'dest': 'public_dataset',
            'default': defaults.get('public_dataset', False),
            'help': "Make generated dataset public."},

        # Set a price tag to your dataset.
        '--dataset-price': {
            'action': 'store',
            'dest': 'dataset_price',
            'type': float,
            'default': defaults.get('dataset_price', 0.0),
            'help': "Price for the dataset."},

        # Doesn't make dataset public.
        '--no-public-dataset': {
            'action': 'store_false',
            'dest': 'public_dataset',
            'default': defaults.get('public_dataset', False),
            'help': "Doesn't make generated dataset public."},

        # Does not create a dataset.
        '--no-dataset': {
            'action': 'store_true',
            'dest': 'no_dataset',
            'default': defaults.get('no_dataset', False),
            'help': "Do not create a dataset."},

        # Seed. The value used in dataset's splits and sampling as seed
        '--seed': {
            'action': 'store',
            'dest': 'seed',
            'default': defaults.get('seed', None),
            'help': "Value used as seed in dataset splits and sampling."},

        # The path to a file containing dataset attributes.
        '--dataset-attributes': {
            'action': 'store',
            'dest': 'dataset_attributes',
            'default': defaults.get('dataset_attributes', None),
            'help': ("Path to a json file describing dataset"
                     " attributes.")},
        # Multi-dataset. Generate new dataset from a list of existing datasets.
        '--multi-dataset': {
            'action': 'store_true',
            'dest': 'multi_dataset',
            'default': defaults.get('multi_dataset', False),
            'help': ("Generate a new dataset by adding existing"
                     " datasets.")},

        # The path to a file containing multi-dataset attributes.
        '--multi-dataset-attributes': {
            'action': 'store',
            'dest': 'multi_dataset_attributes',
            'default': defaults.get('multi_dataset_attributes', None),
            'help': ("Path to a json file describing multi-dataset"
                     " attributes.")},

        # Create a dataset.
        '--no-no-dataset': {
            'action': 'store_false',
            'dest': 'no_dataset',
            'default': defaults.get('no_dataset', False),
            'help': "Create a dataset."},

        # Set the part of training data to be held out for testing
        '--test-split': {
            'action': 'store',
            'dest': 'test_split',
            'type': float,
            'default': defaults.get('test_split', 0.0),
            'help': ("Part of training data to be held out for "
                     "testing (e.g. --test-split 0.2).")},

        # Path to the file containing fields generators for the new dataset.
        # Used when generating a dataset from another by adding new fields
        # combining or setting its contents.
        '--new-fields': {
            'action': 'store',
            'dest': 'new_fields',
            'default': defaults.get('new_fields', None),
            'help': ("Path to the file containing fields generators."
                     " Used to create a new dataset from an existing"
                     " one by adding new fields combining or"
                     " setting its contents.")},

        # Exports the dataset to a CSV file
        '--to-csv': {
            'action': 'store',
            'dest': 'to_csv',
            'nargs': '?',
            'const': '',
            'default': defaults.get('to_csv', None),
            'help': "Path to the exported dataset file."}}

    return options



# -*- coding: utf-8 -*-
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""Options for BigMLer deepnet

"""

def get_deepnet_options(defaults=None):
    """Adding arguments for the deepnet subcommand

    """

    if defaults is None:
        defaults = {}

    options = {
        # Input fields to include in the deepnet.
        '--deepnet-fields': {
            "action": 'store',
            "dest": 'deepnet_fields',
            "default": defaults.get('deepnet_fields', None),
            "help": ("Comma-separated list of input fields"
                     " (predictors) to create the deepnet.")},

        # If a BigML deepnet is provided, the script will
        # use it to generate predictions
        '--deepnet': {
            'action': 'store',
            'dest': 'deepnet',
            'default': defaults.get('deepnet', None),
            'help': "BigML deepnet Id."},

        # The path to a file containing deepnet ids.
        '--deepnets': {
            'action': 'store',
            'dest': 'deepnets',
            'default': defaults.get('deepnets', None),
            'help': ("Path to a file containing deepnets/ids."
                     " One deepnet"
                     " per line (e.g., "
                     "deepnet/50a206a8035d0706dc000376"
                     ").")},

        # If a BigML json file containing a deepnet
        # structure is provided,
        # the script will use it.
        '--deepnet-file': {
            'action': 'store',
            'dest': 'deepnet_file',
            'default': defaults.get('deepnet_file', None),
            'help': "BigML deepnet JSON structure file."},

        # The batch_normalization. Specifies whether to normalize the outputs
        # of a network before being passed to the activation function or not.
        '--batch-normalization': {
            'action': 'store_true',
            'dest': 'batch_normalization',
            'default': defaults.get('batch_normalization', None),
            'help': ("Specifies whether to normalize the outputs of"
                     " a network before being passed to the activation"
                     " function or not.")},

        # Stopping criteria for solver.
        '--default-numeric-value': {
            'action': 'store',
            'dest': 'default_numeric_value',
            'default': defaults.get('default_numeric_value', None),
            'choices': ["mean", "median", "minimum", "maximum", "zero"],
            'help': ("It accepts any of the following strings to substitute"
                     " missing numeric values across all the numeric fields"
                     " in the dataset: 'mean', 'median', 'minimum',"
                     " 'maximum', 'zero'.")},

        # dropout to control overfitting
        '--dropout-rate': {
            'action': 'store',
            'dest': 'dropout_rate',
            'type': float,
            'default': defaults.get('dropout_rate', None),
            'help': ("A number between 0 and 1 specifying the rate at "
                     "which to drop weights during training to control"
                     " overfitting")},

        # Hidden layers description
        '--hidden-layers': {
            'action': 'store',
            'dest': 'hidden_layers',
            'default': defaults.get('hidden_layers', None),
            'help': ("A JSON file that contains a list of maps describing"
                     " the number and type of layers in the network "
                     "(other than the output layer, which is determined by "
                     "the type of learning problem).")},

        # Use alternate layers for residuals
        '--learn-residuals': {
            'action': 'store_true',
            'dest': 'learn_residuals',
            'default': defaults.get('learn_residuals', None),
            'help': ("Specifies whether alternate layers should learn a"
                     " representation of the residuals for a given layer"
                     " rather than the layer itself or not.")},

        # Learning rate
        '--learning-rate': {
            'action': 'store',
            'dest': 'learning_rate',
            'type': float,
            'default': defaults.get('learning_rate', None),
            'help': ("A number between 0 and 1 specifying the"
                     " learning rate.")},

        # Max iterations
        '--max-iterations': {
            'action': 'store',
            'dest': 'max_iterations',
            'type': int,
            'default': defaults.get('max_iterations', None),
            'help': ("A number between 100 and 100000 for the maximum "
                     "number of gradient steps to take during the"
                     " optimization.")},

        # Max training time
        '--max-training-time': {
            'action': 'store',
            'dest': 'max_training_time',
            'type': int,
            'default': defaults.get('max_training_time', None),
            'help': ("The maximum wall-clock training time, in seconds, "
                     "for which to train the network. ")},

        # Number of hidden layers
        '--number-of-hidden-layers': {
            'action': 'store',
            'dest': 'number_of_hidden_layers',
            'type': int,
            'default': defaults.get('number_of_hidden_layers', None),
            'help': ("The number of hidden layers to use in the network. "
                     "If the number is greater than the length of the list"
                     " of hidden_layers, the list is cycled until the"
                     " desired number is reached. If the number is smaller"
                     " than the length of the list of hidden_layers, the"
                     " list is shortened. ")},

        # Number of model candidates
        '--number-of-model-candidates': {
            'action': 'store',
            'dest': 'number_of_model_candidates',
            'type': int,
            'default': defaults.get('number_of_model_candidates', None),
            'help': ("An  integer specifying the number of models to try "
                     "during the model search. ")},

        # use search
        '--search': {
            'action': 'store',
            'dest': 'search',
            'default': defaults.get('search', None),
            'help': ("During the deepnet creation, BigML trains and"
                     " evaluates over all possible network configurations,"
                     " returning the best networks found for the problem. "
                     "The final deepnet returned by the search is a "
                     "compromise between the top n networks found in the"
                     " search. Since this option builds several networks,"
                     " it may be significantly slower than the"
                     " suggest_structure technique.")},

        # Suggest structure
        '--suggest-structure': {
            'action': 'store_true',
            'dest': 'suggest_structure',
            'default': defaults.get('suggest_structure', None),
            'help': ("An alternative to the search technique that is "
                     "usually a more efficient way to quickly train "
                     "and iterate deepnets and it can reach similar"
                     " results. BigML has learned some general rules "
                     "about what makes one network structure better"
                     " than another for a given dataset. Given your"
                     " dataset, BigML will automatically suggest a"
                     " structure and a set of parameter values that"
                     " are likely to perform well for your dataset."
                     " This option only builds one network.")},

        # Missing numeric values are  used
        '--missing-numerics': {
            'action': 'store_true',
            'dest': 'missing_numerics',
            'default': defaults.get('missing_numerics', True),
            'help': ("Whether to create an additional binary predictor each"
                     " numeric field which denotes a missing value. If"
                     " false, these predictors are not created, and rows"
                     " containing missing numeric values are dropped. ")},

        # Missing numeric values are not used
        '--no-missing-numerics': {
            'action': 'store_false',
            'dest': 'missing_numerics',
            'default': defaults.get('missing_numerics', True),
            'help': ("Whether to create an additional binary predictor each"
                     " numeric field which denotes a missing value. If"
                     " true, these predictors are not created, and rows"
                     " containing missing numeric values are dropped. ")},

        # Suggest structure
        '--tree-embedding': {
            'action': 'store_true',
            'dest': 'tree_embedding',
            'default': defaults.get('tree_embedding', None),
            'help': ("Specify whether to learn a tree-based representation"
                     " of the data as engineered features along with the"
                     " raw features, essentially by learning trees over "
                     "slices of the input space and a small amount of "
                     "the training data. The theory is that these engineered"
                     " features will linearize obvious non-linear "
                     "dependencies before training begins, and so make "
                     "learning proceed more quickly. ")},

        # Does not balance fields
        '--no-balance-fields': {
            'action': 'store_false',
            'dest': 'balance_fields',
            'default': defaults.get('balance_fields', True),
            'help': "Do not balance fields."},

        # Does not create a deepnet just a dataset.
        '--no-deepnet': {
            'action': 'store_true',
            'dest': 'no_deepnet',
            'default': defaults.get('no_deepnet', False),
            'help': "Do not create a deepnet."},

        # The path to a file containing deepnet attributes.
        '--deepnet-attributes': {
            'action': 'store',
            'dest': 'deepnet_attributes',
            'default': defaults.get('deepnet_attributes', None),
            'help': ("Path to a json file describing deepnet"
                     " attributes.")},

        # Create a deepnet, not just a dataset.
        '--no-no-depnet': {
            'action': 'store_false',
            'dest': 'no_deepnet',
            'default': defaults.get('no_deepnet', False),
            'help': "Create a deepnet."}}

    return options



# -*- coding: utf-8 -*-
#
# Copyright 2016-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Logistic regression prediction auxiliary functions

"""
from __future__ import absolute_import


import sys

import bigml.api

from bigml.logistic import LogisticRegression
from bigml.io import UnicodeWriter

import bigmler.utils as u
import bigmler.checkpoint as c

from bigmler.tst_reader import TstReader as TestReader
from bigmler.resources import NORMAL_FORMAT, FULL_FORMAT
from bigmler.resources import create_batch_prediction
from bigmler.prediction import use_prediction_headers


def write_prediction(prediction, output=sys.stdout,
                     prediction_info=NORMAL_FORMAT, input_data=None,
                     exclude=None):
    """Writes the final prediction to the required output

       The format of the output depends on the `prediction_info` value.
       There's a brief format, that writes only the predicted value,
       and a full data format that writes first the input data
       used to predict followed by the prediction.

    """

    row = []
    # input data is added if prediction format is BRIEF (no probability) or
    # FULL
    if prediction_info != NORMAL_FORMAT:
        if input_data is None:
            input_data = []
        row = input_data
        if exclude:
            for index in exclude:
                del row[index]
    row.append(prediction.get('prediction'))
    if prediction_info in [NORMAL_FORMAT, FULL_FORMAT]:
        row.append(prediction.get('probability'))
    try:
        output.writerow(row)
    except AttributeError:
        try:
            output.write(row)
        except AttributeError:
            raise AttributeError("You should provide a writeable object")


def local_prediction(logistic_regressions, test_reader, output, args,
                     exclude=None):
    """Get local logistic_regression and issue prediction

    """
    # Only one logistic_regression at present
    local_logistic = LogisticRegression(logistic_regressions[0],
                                        api=args.retrieve_api_)
    kwargs = {"full": True}
    if args.operating_point_:
        kwargs.update({"operating_point": args.operating_point_})
    for input_data in test_reader:
        input_data_dict = test_reader.dict(input_data, filtering=False)
        prediction_info = local_logistic.predict(
            input_data_dict, **kwargs)
        write_prediction(prediction_info, output,
                         args.prediction_info, input_data, exclude)


def lr_prediction(logistic_regressions, fields, args, session_file=None):
    """Computes a logistic regression prediction
    for each entry in the `test_set`.

    """
    test_set = args.test_set
    test_set_header = args.test_header
    output = args.predictions
    test_reader = TestReader(test_set, test_set_header, fields,
                             None,
                             test_separator=args.test_separator)
    with UnicodeWriter(output, lineterminator="\n") as output:
        # columns to exclude if input_data is added to the prediction field
        exclude = use_prediction_headers(
            args.prediction_header, output, test_reader, fields, args,
            args.objective_field, quality="probability")

        # Local predictions: Predictions are computed locally
        message = u.dated("Creating local predictions.\n")
        u.log_message(message, log_file=session_file, console=args.verbosity)
        local_prediction(logistic_regressions, test_reader,
                         output, args, exclude=exclude)
    test_reader.close()


def remote_lr_prediction(logistic_regression, test_dataset,
                         batch_prediction_args, args,
                         api, resume, prediction_file=None, session_file=None,
                         path=None, log=None):
    """Computes a prediction for each entry in the `test_set`.

       Predictions are computed remotely using the batch prediction call.
    """

    logistic_regression_id = bigml.api.get_logistic_regression_id( \
        logistic_regression)
    # if resuming, try to extract dataset form log files
    if resume:
        message = u.dated("Batch prediction not found. Resuming.\n")
        resume, batch_prediction = c.checkpoint(
            c.is_batch_prediction_created, path, debug=args.debug,
            message=message, log_file=session_file, console=args.verbosity)
    if not resume:
        batch_prediction = create_batch_prediction(
            logistic_regression_id, test_dataset, batch_prediction_args,
            args, api, session_file=session_file, path=path, log=log)
    if not args.no_csv:
        file_name = api.download_batch_prediction(batch_prediction,
                                                  prediction_file)
        if file_name is None:
            sys.exit("Failed downloading CSV.")
    if args.to_dataset:
        batch_prediction = bigml.api.check_resource(batch_prediction, api=api)
        new_dataset = bigml.api.get_dataset_id(
            batch_prediction['object']['output_dataset_resource'])
        if new_dataset is not None:
            message = u.dated("Batch prediction dataset created: %s\n"
                              % u.get_url(new_dataset))
            u.log_message(message, log_file=session_file,
                          console=args.verbosity)
            u.log_created_resources("batch_prediction_dataset",
                                    path, new_dataset, mode='a')



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2015-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing export model generation

"""
from __future__ import absolute_import


from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, teardown_class)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.export_steps as export

def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestExport(object):

    def teardown(self):
        """Calling generic teardown for every method

        """
        print "\nEnd of tests in: %s\n-------------------\n" % __name__
        teardown_class()

    def setup(self):
        """
            Debug information
        """
        print "\n-------------------\nTests in: %s\n" % __name__

    def test_scenario1(self):
        """
            Scenario: Successfully exporting models with params in the available languages:
                Given I create BigML resources uploading train "<data>" file using "<source_attributes>" and log in "<output>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the model has been created
                And I export the model as a function in "<language>"to "<output>"
                Then the export file is like "<check_file>"

                Examples:
                | data                 | source_attributes             | output                 | language       | check_file
                | ../data/movies.csv   | data/movies_source_attrs.json | ./scenario_exp_1/model | python         | model_function.py

        """
        print self.test_scenario1.__doc__
        examples = [
            ['data/movies.csv', 'data/movies_source_attrs.json', 'scenario_exp_1_a/model', 'python', 'check_files/export/model_function.py'],
            ['data/movies.csv', 'data/movies_source_attrs.json', 'scenario_exp_1_b/model', 'javascript', 'check_files/export/model_function.js'],
            ['data/movies.csv', 'data/movies_source_attrs.json', 'scenario_exp_1_c/model', 'r', 'check_files/export/model_function.R'],
            ['data/iris.csv', '', 'scenario_exp_1_d/model', 'tableau', 'check_files/export/model_function.tb'],
            ['data/iris.csv', '', 'scenario_exp_1_e/model', 'mysql', 'check_files/export/model_function.sql'],
            ['data/libros.csv', 'data/libros_source_attrs.json', 'scenario_exp_1_f/model', 'python', 'check_files/export/model_function_utf8.py'],
            ['data/libros.csv', 'data/libros_source_attrs.json', 'scenario_exp_1_g/model', 'r', 'check_files/export/model_function_utf8.R'],
            ['data/libros.csv', 'data/libros_source_attrs.json', 'scenario_exp_1_h/model', 'javascript', 'check_files/export/model_function_utf8.js']]
        for example in examples:
            print "\nTesting with:\n", example
            export.i_create_all_resources_to_model_with_source_attrs( \
                self, data=example[0], source_attributes=example[1], output=example[2])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self)
            test_pred.i_check_create_model(self)
            export.i_export_model(self, language=example[3], output=example[2])
            export.i_check_if_the_output_is_like_expected_file( \
                self, language=example[3], expected_file=example[4])



# -*- coding: utf-8 -*-
#
# Copyright 2014-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""Options for BigMLer logistic regression

"""

def get_logistic_regression_options(defaults=None):
    """Adding arguments for the logistic regression subcommand

    """

    if defaults is None:
        defaults = {}

    options = {
        # Input fields to include in the logistic regression.
        '--logistic-fields': {
            "action": 'store',
            "dest": 'logistic_fields',
            "default": defaults.get('logistic_fields', None),
            "help": ("Comma-separated list of input fields"
                     " (predictors) to create the logistic regression.")},

        # If a BigML logistic regression is provided, the script will
        # use it to generate predictions
        '--logistic-regression': {
            'action': 'store',
            'dest': 'logistic_regression',
            'default': defaults.get('logistic_regression', None),
            'help': "BigML logistic regression Id."},

        # The path to a file containing logistic regression ids.
        '--logistic-regressions': {
            'action': 'store',
            'dest': 'logistic_regressions',
            'default': defaults.get('logistic_regressions', None),
            'help': ("Path to a file containing logisticregression/ids."
                     " One logisticregression"
                     " per line (e.g., "
                     "logisticregression/50a206a8035d0706dc000376"
                     ").")},

        # If a BigML json file containing a logistic regression
        # structure is provided,
        # the script will use it.
        '--logistic-file': {
            'action': 'store',
            'dest': 'logistic_file',
            'default': defaults.get('logistic_file', None),
            'help': "BigML logistic regression JSON structure file."},

        # bias term in the logistic regression.
        '--bias': {
            "action": 'store_true',
            "dest": 'bias',
            "default": defaults.get('bias', True),
            "help": ("Use intercept term. Will"
                     " include an intercept term in the solution.")},

        # Don't include a bias term in the logistic regression.
        '--no-bias': {
            "action": 'store_false',
            "dest": 'bias',
            "default": defaults.get('bias', True),
            "help": ("Do not use intercept term. It will"
                     " exclude the intercept term from the solution.")},

        # The strength of the regularization step. Must be greater than 0.
        '--c': {
            'action': 'store',
            'dest': 'lr_c',
            'default': defaults.get('lr_c', 1),
            'type': float,
            'help': ("The strength of the regularization step. Must be"
                     " greater than 0.")},

        # Stopping criteria for solver.
        '--eps': {
            'action': 'store',
            'dest': 'eps',
            'default': defaults.get('eps', 0.00001),
            'type': float,
            'help': ("Stopping criteria for solver. If the difference between"
                     " the results from the current and last iterations is"
                     " less than eps, then the solver is finished.")},

        # Field codings for categorical fields
        '--field-codings': {
            'action': 'store',
            'dest': 'field_codings',
            'default': defaults.get('field_codings', None),
            'help': ("Path to a JSON file with coding schemes for"
                     " categorical fields: dummy,"
                     " contrast, or other. Its contents is a list of maps "
                     " with field identifiers and a coding scheme for"
                     " that field")},

        # No missing values for numeric predictors
        '--no-missing-numerics': {
            'action': 'store_false',
            'dest': 'missing_numerics',
            'default': defaults.get('missing_numerics', True),
            'help': ("Whether to create an additional binary predictor each"
                     " numeric field which denotes a missing value. If"
                     " true, these predictors are not created, and rows"
                     " containing missing numeric values are dropped. ")},

        # Missing values for numeric predictors
        '--normalize': {
            'action': 'store_true',
            'dest': 'normalize',
            'default': defaults.get('normalize', False),
            'help': ("Whether to normalize feature vectors in training"
                     " and predicting.")},

        # Does not balance fields
        '--no-balance-fields': {
            'action': 'store_false',
            'dest': 'balance_fields',
            'default': defaults.get('balance_fields', True),
            'help': "Do not balance fields."},

        # Does not create a logistic regression just a dataset.
        '--no-logistic-regression': {
            'action': 'store_true',
            'dest': 'no_logistic_regression',
            'default': defaults.get('no_logistic_regression', False),
            'help': "Do not create a logistic regression."},

        # The path to a file containing logistic regression attributes.
        '--logistic-regression-attributes': {
            'action': 'store',
            'dest': 'logistic_regression_attributes',
            'default': defaults.get('logistic_regression_attributes', None),
            'help': ("Path to a json file describing logistic regression"
                     " attributes.")},

        # Create a logistic regression, not just a dataset.
        '--no-no-logistic-regression': {
            'action': 'store_false',
            'dest': 'no_logistic_regression',
            'default': defaults.get('no_logistic_regression', False),
            'help': "Create a logistic."}}

    return options



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2015-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing projects

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, teardown_class)

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.project_creation_steps as test_project
import bigmler.tests.delete_subcommand_steps as delete


def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestProjects(object):

    def teardown(self):
        """Calling generic teardown for every method

        """
        print "\nEnd of tests in: %s\n-------------------\n" % __name__
        teardown_class()

    def setup(self):
        """
            Debug information
        """
        print "\n-------------------\nTests in: %s\n" % __name__

    def test_scenario1(self):
        """
            Scenario: Successfully creating a project and associating resources to it:
                Given I create a BigML source uploading train "<data>" file and associate it to a new project named "<project>" storing results in "<output_dir>"
                And I check that the project has been created
                And I check that the source has been created
                Then the source is associated to the project

                Examples:
                | data             | project         | output_dir
                | ../data/iris.csv | My new project  | ./scenario_p_1
        """
        print self.test_scenario1.__doc__
        examples = [
            ['data/iris.csv', 'My new project', 'scenario_p_1']]
        for example in examples:
            print "\nTesting with:\n", example
            test_project.i_create_source_with_project(self, data=example[0], project=example[1], output_dir=example[2])
            test_project.i_check_create_project(self)
            test_pred.i_check_create_source(self)
            test_project.check_source_in_project(self)

    def test_scenario2(self):
        """
            Scenario: Successfully associating resources to an existing project:
                Given I create a BigML source uploading train "<data>" file and associate it to a new project named "<project>" storing results in "<output_dir>"
                And I check that the project has been created
                And I check that the source has been created
                And I create a BigML source uploading train "<data>" file and associate it to the last created project id storing results in "<output_dir2>"
                Then the source is associated to the project

                Examples:
                | data             | project         | output_dir     | output_dir2
                | ../data/iris.csv | My new project  | ./scenario_p_2 | ./scenario_p_2_1
        """
        print self.test_scenario2.__doc__
        examples = [
            ['data/iris.csv', 'My new project', 'scenario_p_2', 'scenario_p_2_1']]
        for example in examples:
            print "\nTesting with:\n", example
            test_project.i_create_source_with_project(self, data=example[0], project=example[1], output_dir=example[2])
            test_project.i_check_create_project(self)
            test_pred.i_check_create_source(self)
            test_project.i_create_source_with_project_id(self, data=example[0], output_dir=example[3])
            test_project.check_source_in_project(self)

    def test_scenario3(self):
        """
            Scenario: Successfully creating resources with no project association:
                Given I create a BigML source from file "<data>" storing results in "<output_dir>"
                And I check that the source has been created
                Then the source has no project association

                Examples:
                | data             | output_dir
                | ../data/iris.csv | ./scenario_p_3
        """
        print self.test_scenario3.__doc__
        examples = [
            ['data/iris.csv', 'scenario_p_3']]
        for example in examples:
            print "\nTesting with:\n", example
            delete.i_create_source_from_file(self, data=example[0], output_dir=example[1])
            test_pred.i_check_create_source(self)
            test_project.check_source_in_no_project(self)

    def test_scenario4(self):
        """
            Scenario: Successfully creating and updating project:
                Given I create a BigML project "<project>" and log results in "<output_dir>"
                And I check that the project has been created
                And I update the project params "<params>" to "<values>"
                And I check that the project has been updated
                Then the project params "<params>" are "<values>"

                Examples:
                | project         | output_dir       | params         | values
                | My new project  | ./scenario_p_4   | name, category | my_project_name, 1
        """
        print self.test_scenario4.__doc__
        examples = [
            ['My new project', 'scenario_p_4', ['name', 'category'], ['my_project_name', 1]]]
        for example in examples:
            print "\nTesting with:\n", example
            test_project.i_create_project(self, project=example[0], output_dir=example[1])
            test_project.i_check_create_project(self)
            test_project.i_update_project(self, params=example[2], values=example[3])
            test_project.i_check_update_project(self)
            test_project.check_params_values(self, params=example[2], values=example[3])



# -*- coding: utf-8 -*-
#
# Copyright 2014-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import absolute_import


import os
import time
import csv
import json
import sys


from bigmler.tests.world import world, res_filename
from subprocess import check_call, CalledProcessError
from bigml.api import check_resource
from bigmler.checkpoint import file_number_of_lines
from bigmler.tests.common_steps import check_debug
from bigmler.tests.basic_tst_prediction_steps import shell_execute
from bigmler.utils import PYTHON3

CAT = u"type " if sys.platform == "win32" else u"cat "

#@step(r'I create BigML resources uploading train "(.*)" file to test "(.*)" read from stdin with name "(.*)" and log predictions in "(.*)"$')
def i_create_all_resources_to_test_from_stdin(step, data=None, test=None, name=None, output=None):
    if data is None or test is None or output is None or name is None:
        assert False

    test = res_filename(test)
    if not PYTHON3:
        name = name.decode("utf-8")

    command = (CAT + test + u"|bigmler --train " + res_filename(data) +
               u" --test --store --output " + output + u" --name \"" +
               name + u"\" --max-batch-models 1")
    shell_execute(command, output, test=test)


#@step(r'I create a BigML source from stdin using train "(.*)" file and logging in "(.*)"$')
def i_create_source_from_stdin(step, data=None, output_dir=None):
    if data is None or output_dir is None:
        assert False
    command = (CAT + res_filename(data) + u"|bigmler --train " +
               u"--store --no-dataset --no-model --output-dir " +
               output_dir + u" --max-batch-models 1")
    shell_execute(command, output_dir + "/test", test=None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

count = 0
previous = None

def print_result(values, prediction, count):
    """Prints input data and predicted value as an ordered list.

    """
    result = "[%s, %s]" % (values, prediction)
    print u"%s\\t%s" % (result, count)

for line in sys.stdin:
    values, prediction = line.strip().split('\\t')
    if previous is None:
        previous = (values, prediction)
    if values != previous[0]:
        print_result(previous[0], previous[1], count)
        previous = (values, prediction)
        count = 0
    count += 1
if count > 0:
    print_result(previous[0], previous[1], count)



# -*- coding: utf-8 -*-
#
# Copyright 2014-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - Resources processing: creation, update and retrieval of models

"""
from __future__ import absolute_import

import copy

import bigml.api

from bigml.fields import Fields, DEFAULT_MISSING_TOKENS

import bigmler.utils as u
import bigmler.resources as r
import bigmler.checkpoint as c

from bigmler.processing.ensembles import (ensemble_processing,
                                          ensemble_per_label)

MONTECARLO_FACTOR = 200


def model_per_label(labels, datasets, api, args, resume, fields=None,
                    multi_label_data=None,
                    session_file=None, path=None, log=None):
    """Creates a model per label for multi-label datasets

    """
    model_ids = []
    models = []
    args.number_of_models = len(labels)
    if resume:
        resume, model_ids = c.checkpoint(
            c.are_models_created, path, args.number_of_models,
            debug=args.debug)
        if not resume:
            message = u.dated("Found %s models out of %s."
                              " Resuming.\n"
                              % (len(model_ids),
                                 args.number_of_models))
            u.log_message(message, log_file=session_file,
                          console=args.verbosity)

        models = model_ids
    args.number_of_models = len(labels) - len(model_ids)
    model_args_list = r.set_label_model_args(args, fields, labels,
                                             multi_label_data)

    # create models changing the input_field to select
    # only one label at a time
    models, model_ids = r.create_models(
        datasets, models, model_args_list, args, api,
        path, session_file, log)
    args.number_of_models = 1
    return models, model_ids, resume


def models_processing(datasets, models, model_ids,
                      api, args, resume, fields=None,
                      session_file=None, path=None,
                      log=None, labels=None, multi_label_data=None,
                      other_label=None):
    """Creates or retrieves models from the input data

    """
    ensemble_ids = []

    # If we have a dataset but not a model, we create the model if the no_model
    # flag hasn't been set up.
    if datasets and not (args.has_models_ or args.no_model):
        dataset = datasets[0]
        model_ids = []
        models = []
        if args.multi_label:
            # If --number-of-models is not set or is 1, and there's
            # no boosting options on, create one model per
            # label. Otherwise, create one ensemble per label with the required
            # number of models
            if args.number_of_models < 2 and not args.boosting:
                models, model_ids, resume = model_per_label(
                    labels, datasets, api, args, resume,
                    fields=fields, multi_label_data=multi_label_data,
                    session_file=session_file, path=path, log=log)
            else:
                (ensembles, ensemble_ids,
                 models, model_ids, resume) = ensemble_per_label(
                     labels, dataset, api, args, resume, fields=fields,
                     multi_label_data=multi_label_data,
                     session_file=session_file, path=path, log=log)

        elif args.number_of_models > 1 or args.boosting:
            ensembles = []
            # Ensembles of models
            (ensembles, ensemble_ids,
             models, model_ids, resume) = ensemble_processing(
                 datasets, api, args, resume,
                 fields=fields,
                 session_file=session_file, path=path, log=log)
            ensemble = ensembles[0]
            args.ensemble = bigml.api.get_ensemble_id(ensemble)

        else:
            # Set of partial datasets created setting args.max_categories
            if len(datasets) > 1 and args.max_categories:
                args.number_of_models = len(datasets)
            if ((args.test_datasets and args.evaluate) or
                    (args.datasets and args.evaluate and args.dataset_off)):
                args.number_of_models = len(args.dataset_ids)
            # Cross-validation case: we create 2 * n models to be validated
            # holding out an n% of data
            if args.cross_validation_rate > 0:
                if args.number_of_evaluations > 0:
                    args.number_of_models = args.number_of_evaluations
                else:
                    args.number_of_models = int(MONTECARLO_FACTOR *
                                                args.cross_validation_rate)
            if resume:
                resume, model_ids = c.checkpoint(
                    c.are_models_created, path, args.number_of_models,
                    debug=args.debug)
                if not resume:
                    message = u.dated("Found %s models out of %s. Resuming.\n"
                                      % (len(model_ids),
                                         args.number_of_models))
                    u.log_message(message, log_file=session_file,
                                  console=args.verbosity)

                models = model_ids
                args.number_of_models -= len(model_ids)
            model_args = r.set_model_args(args,
                                          fields=fields,
                                          objective_id=args.objective_id_,
                                          model_fields=args.model_fields_,
                                          other_label=other_label)
            models, model_ids = r.create_models(datasets, models,
                                                model_args, args, api,
                                                path, session_file, log)
    # If a model is provided, we use it.
    elif args.model:
        model_ids = [args.model]
        models = model_ids[:]

    elif args.models or args.model_tag:
        models = model_ids[:]

    if args.ensemble:
        if not args.ensemble in ensemble_ids:
            ensemble_ids.append(args.ensemble)
        if not args.evaluate:
            ensemble = r.get_ensemble(args.ensemble, api, args.verbosity,
                                      session_file)
            model_ids = ensemble['object']['models']
            models = model_ids[:]

    if args.ensembles or args.ensemble_tag:
        model_ids = []
        ensemble_ids = []
        # Parses ensemble/ids if provided.
        if args.ensemble_tag:
            ensemble_ids = (ensemble_ids +
                            u.list_ids(api.list_ensembles,
                                       "tags__in=%s" % args.ensemble_tag))
        else:
            ensemble_ids = u.read_resources(args.ensembles)
        for ensemble_id in ensemble_ids:
            ensemble = r.get_ensemble(ensemble_id, api)
            if args.ensemble is None:
                args.ensemble = ensemble_id
            model_ids.extend(ensemble['object']['models'])
        models = model_ids[:]

    # If we are going to predict we must retrieve the models
    if model_ids and args.test_set and not args.evaluate:
        models, model_ids = r.get_models(models, args, api, session_file)

    return models, model_ids, ensemble_ids, resume


def get_model_fields(model, csv_properties, args, single_model=True,
                     multi_label_data=None):
    """Retrieves fields info from model resource

    """
    if not csv_properties:
        csv_properties = {}
    csv_properties.update(verbose=True)
    if args.user_locale is None:
        args.user_locale = model['object'].get('locale', None)
    csv_properties.update(data_locale=args.user_locale)
    if (single_model and 'model_fields' in model['object']['model'] and
            args.test_header):
        model_fields = model['object']['model']['model_fields'].keys()
        csv_properties.update(include=model_fields)
    else:
        csv_properties.update(include=None)
    if 'missing_tokens' in model['object']['model']:
        missing_tokens = model['object']['model']['missing_tokens']
    else:
        missing_tokens = DEFAULT_MISSING_TOKENS
    csv_properties.update(missing_tokens=missing_tokens)
    # if the model belongs to a multi-label set of models, the real objective
    # field is never amongst the set of fields of each individual model, so
    # we must add it.
    fields_dict = copy.deepcopy(model['object']['model']['fields'])

    if args.multi_label:
        # Adds the real objective field to fields_dict
        objective_field = multi_label_data['objective_name']
        objective_id = multi_label_data['objective_id']
        objective_column = multi_label_data['objective_column']
        fields_dict[objective_id] = {"op_type": "categorical",
                                     "name": objective_field,
                                     "column_number": objective_column}
    else:
        objective_field = model['object']['objective_fields']
        if isinstance(objective_field, list):
            objective_field = objective_field[0]
    csv_properties.update(objective_field=objective_field)
    args.objective_field = objective_field
    return Fields(fields_dict, **csv_properties)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2014 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Tree level output in R

This module defines functions that generate R output for the tree
"""

from bigml.tree_utils import (sort_fields, filter_nodes, missing_branch,
                              split, INDENT, PYTHON_OPERATOR, COMPOSED_FIELDS,
                              NUMERIC_VALUE_FIELDS)

from bigml.tree import Tree


def value_to_print(value, optype):
    """String of code that represents a value according to its type

    """
    if value is None:
        return "NA"
    if optype in NUMERIC_VALUE_FIELDS:
        return value
    return "\"%s\"" % value.replace('"', '\\"')


class RTree(Tree):

    def missing_check_code(self, field, depth, cmv, metric):
        """Builds the code to predict when the field is missing

        """

        code = u"%sif (is.na(%s)){\n" % \
               (INDENT * depth,
                self.fields[field]['dotted'])
        value = value_to_print(self.output,
                               self.fields[self.objective_id]['optype'])
        indent_depth = INDENT * (depth + 1)
        code += u"%sreturn(list(prediction=%s, %s=%s))\n%s}\n" % \
               (indent_depth,
                value, metric, self.confidence, INDENT * depth)
        cmv.append(self.fields[field]['dotted'])
        return code

    def missing_prefix_code(self, field, cmv):
        """Part of the condition that checks for missings when missing_splits
        has been used

        """

        operator = u"==" if self.predicate.missing else u"!="
        connection = u"||" if self.predicate.missing else u"&&"
        if not self.predicate.missing:
            cmv.append(self.fields[field]['dotted'])
        return u"%s%s %s NA %s " % (self.fields[field]['dotted'],
                                    operator,
                                    connection)

    def split_condition_code(self, field, depth,
                             pre_condition, term_analysis_fields,
                             item_analysis_fields):
        """Condition code for the split

        """
        optype = self.fields[field]['optype']
        operator = PYTHON_OPERATOR[self.predicate.operator]
        value = value_to_print(self.predicate.value, optype)

        if optype in COMPOSED_FIELDS:
            if optype == 'text':
                term_analysis_fields.append((field,
                                             self.predicate.term))
                matching_function = "termMatches"
            else:
                item_analysis_fields.append((field,
                                             self.predicate.term))
                matching_function = "itemMatches"

            return u"%sif (%s%s(%s, %s, %s)%s%s) {\n" % \
                (INDENT * depth, pre_condition, matching_function,
                 self.fields[field]['dotted'],
                 value_to_print(self.fields[field]['dotted'], 'categorical'),
                 value_to_print(self.predicate.term, 'categorical'),
                 operator,
                 value)
        if self.predicate.value is None:
            cmv.append(self.fields[field]['dotted'])
        return u"%sif (%s%s %s %s) {\n" % \
            (INDENT * depth, pre_condition,
             self.fields[field]['dotted'],
             operator,
             value)

    def plug_in_body(self, depth=1, cmv=None, ids_path=None, subtree=True):
        """Translate the model into a set of "if" r statements.

        `depth` controls the size of indentation. If `cmv` (control missing
        values) is set to True then as soon as a value is missing to
        evaluate a predicate the output at that node is returned without
        further evaluation.

        """
        metric = "error" if self.regression else "confidence"
        if cmv is None:
            cmv = []
        body = u""
        term_analysis_fields = []
        item_analysis_fields = []
        field_obj = self.fields[self.objective_id]
        children = filter_nodes(self.children, ids=ids_path,
                                subtree=subtree)

        if children:
            # field used in the split
            field = split(children)

            has_missing_branch = missing_branch(children)
            # the missing is singled out as a special case only when there's
            # no missing branch in the children list
            one_branch = not (has_missing_branch or
                self.fields[field]['optype'] in COMPOSED_FIELDS)
            if (one_branch and
                    not self.fields[field]['dotted'] in cmv):
                body += self.missing_check_code(field, depth, cmv,
                                                metric)

            for child in self.children:
                field = child.predicate.field
                pre_condition = u""
                # code when missing_splits has been used
                if has_missing_branch and field not in COMPOSED_FIELDS \
                        and child.predicate.value is not None:
                    pre_condition = self.missing_prefix_code(child, field, cmv)

                # complete split condition code
                body += child.split_condition_code( \
                    field, depth, pre_condition,
                    term_analysis_fields, item_analysis_fields)

                # value to be determined in next node
                next_level = child.plug_in_body(depth + 1, cmv=cmv[:],
                                                ids_path=ids_path,
                                                subtree=subtree)
                body += next_level[0]
                body += u"%s}\n" % (INDENT * depth)
                term_analysis_fields.extend(next_level[1])
                item_analysis_fields.extend(next_level[2])
        else:
            value = value_to_print(self.output,
                                   self.fields[self.objective_id]['optype'])
            body = u"%sreturn(list(prediction=%s, %s=%s))\n" % \
                  (INDENT * depth,
                   value, metric, self.confidence)
        return body, term_analysis_fields, item_analysis_fields



# -*- coding: utf-8 -*-
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""Options for BigMLer retrain option

"""

def get_retrain_options(defaults=None):
    """Retrain-related options

    """

    if defaults is None:
        defaults = {}
    options = {

        # Resource ID
        '--id': {
            'dest': 'resource_id',
            'default': defaults.get('resource_id', None),
            'help': ("ID for the resource to be retrained.")},
        # path to the data file to be added
        '--add': {
            'dest': 'add',
            'default': defaults.get('add', None),
            'help': ("Path to the data file to be added.")},
        # maximum number of datasets to be used when retraining
        '--window-size': {
            'type': int,
            'dest': 'window_size',
            'default': defaults.get('window_size', -1),
            'help': ("Maximum number of datasets to be used in retraining."
                     " When not set, the new dataset will be added to the"
                     " last one used.")}
    }

    return options



# -*- coding: utf-8 -*-
#
# Copyright 2015-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - reports processing dispatching

"""
from __future__ import absolute_import

import sys
import os
import SimpleHTTPServer
import thread
import webbrowser
import socket

import bigmler.processing.args as a

from bigmler.command import command_handling
from bigmler.reports import (evaluations_report, SERVER_DIRECTORY,
                             HOME, REPORTS_DIR, ANALYZE_TEMPLATE,
                             ANALYZE_DIR)
from bigmler.report.httpserver import StoppableHTTPServer
from bigmler.options.report import DEFAULT_PORT

COMMAND_LOG = u".bigmler_report"

DEFAULT_HOST = "127.0.0.1"


def get_report_url(args):
    """Checks whether there exist reports in the selected directory and creates
       them if they are not there.

    """
    try:
        with open(os.path.join(args.from_dir, REPORTS_DIR, "symlink")) as lnk:
            symlink = lnk.read()
            # currently there's only one template type
            return os.path.join(os.path.basename(symlink),
                                ANALYZE_DIR,
                                os.path.basename(ANALYZE_TEMPLATE))
    except IOError:
        return evaluations_report(args)


def report_dispatcher(args=sys.argv[1:]):
    """Parses command line and calls the different report functions

    """

    command = command_handling(args, COMMAND_LOG)

    # Parses command line arguments.
    command_args = a.parse_and_check(command)

    port = DEFAULT_PORT if not command_args.port else command_args.port
    report_url = get_report_url(command_args)
    if not command_args.no_server:
        absolute_report_url = "http://%s:%s/%s" % (DEFAULT_HOST,
                                                   port, report_url)
        current_directory = os.getcwd()
        os.chdir(os.path.join(HOME, SERVER_DIRECTORY))
        httpd = None
        try:
            httpd = StoppableHTTPServer(\
                (DEFAULT_HOST, port),
                SimpleHTTPServer.SimpleHTTPRequestHandler)
            thread.start_new_thread(httpd.serve, ())
        except socket.error, exc:
            print exc
        # Open URL in new browser window
        webbrowser.open_new(absolute_report_url)
        # opens in default browser
        if httpd:
            raw_input("*********************************\n"
                      "Press <RETURN> to stop the server\n"
                      "*********************************\n")
        os.chdir(current_directory)
        if httpd:
            httpd.stop()



# -*- coding: utf-8 -*-
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import absolute_import

import os
import time
import json
from bigmler.tests.world import world, res_filename
from subprocess import check_call, CalledProcessError
from bigml.api import check_resource
from bigml.io import UnicodeReader
from bigmler.checkpoint import file_number_of_lines
from bigmler.utils import storage_file_name, open_mode, decode2
from bigmler.utils import PYTHON3
from bigmler.tests.common_steps import check_debug
from nose.tools import ok_, assert_equal, assert_not_equal, assert_almost_equal


def shell_execute(command, output, test=None, options=None,
                  test_rows=None, project=True):
    """Excute bigmler command in shell

    """
    command = check_debug(command, project=project)
    world.directory = os.path.dirname(output)
    world.folders.append(world.directory)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        if test is not None:
            world.test_lines = file_number_of_lines(test)
            if options is None or \
                    options.find('--prediction-header') == -1:
                # test file has headers in it, so first line must be ignored
                world.test_lines -= 1
        elif test_rows is not None:
            world.test_lines = test_rows
            if options is not None and \
                    options.find('--prediction-header') > -1:
                world.test_lines += 1
        elif options is not None and \
                options.find('--prediction-header') > -1:
            world.test_lines += 1
        world.output = output
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)


#@step(r'I create BigML time series resources uploading train "(.*?)"'
#       ' file with no headers and log forecasts in "([^"]*)"$')
def i_create_all_ts_resources_with_no_headers( \
    step, data=None, output=None):
    ok_(data is not None and output is not None)
    command = ("bigmler time-series --train " + res_filename(data) +
               " --store --output " + output +
               " --no-train-header --no-test-header")
    shell_execute(command, output, test=test, options='--prediction-header')


#@step(r'I create BigML time series resources uploading train "(.*?)"'
#       ' file to test "(.*?)" and log forecasts in "([^"]*)"$')
def i_create_all_resources(step, data=None, test=None, output=None):
    ok_(data is not None and test is not None and output is not None)
    test = res_filename(test)
    command = ("bigmler --train " + res_filename(data) + " --test " + test +
               " --store --output " + output)
    shell_execute(command, output, test=test)



# -*- coding: utf-8 -*-
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - generates code to use models locally

"""
from __future__ import absolute_import

import sys
import os
import shutil


import bigmler.processing.args as a
import bigmler.utils as u


from bigml.ensemble import Ensemble

from bigmler.defaults import DEFAULTS_FILE
from bigmler.command import get_context
from bigmler.dispatcher import SESSIONS_LOG, clear_log_files
from bigmler.export.out_model.jsmodel import JsModel
try:
    from bigml.out_model.pythonmodel import PythonModel
except:
    from bigml.model import Model as PythonModel
from bigmler.export.out_model.tableaumodel import TableauModel
from bigmler.export.out_model.mysqlmodel import MySQLModel
from bigmler.export.out_model.rmodel import RModel
from bigmler.export.out_model.pythonlr import PythonLR


COMMAND_LOG = u".bigmler_export"
DIRS_LOG = u".bigmler_export_dir_stack"
LOG_FILES = [COMMAND_LOG, DIRS_LOG, u.NEW_DIRS_LOG]

EXPORTS = {
    "javascript": JsModel,
    "python": PythonModel,
    "tableau": TableauModel,
    "mysql": MySQLModel,
    "r": RModel}

EXTENSIONS = {
    "javascript": "js",
    "python": "py",
    "tableau": "tb",
    "mysql": "sql",
    "r": "R"}

LR_EXPORTS   = {
    "python": PythonLR
}

SEPARATE_OUTPUT = ['tableau', 'mysql']

SETTINGS = {
    "command_log": COMMAND_LOG,
    "sessions_log": SESSIONS_LOG,
    "dirs_log": DIRS_LOG,
    "defaults_file": DEFAULTS_FILE}


def export_dispatcher(args=sys.argv[1:]):
    """Parses command line and calls the different export functions

    """

    # If --clear-logs the log files are cleared
    if "--clear-logs" in args:
        clear_log_files(LOG_FILES)

    command_args, command, api, session_file, resume = get_context(args,
                                                                   SETTINGS)
    # Creates the corresponding api instance
    resource = command_args.ensemble or command_args.model
    message = "Generating %s code for %s\n\n" % (command_args.language,
                                                 resource)
    u.log_message(message, \
        log_file=session_file, console=command_args.verbosity)
    export_code(command_args, api)
    u.log_message("_" * 80 + "\n", log_file=session_file)

    u.print_generated_files(command_args.output_dir, log_file=session_file,
                            verbosity=command_args.verbosity)


def generate_output(local_model, args, model_type="model", attr="confidence"):
    """Generates the output for the prediction (and confidence) function

    """
    with open(os.path.join(args.output_dir, \
        "%s.%s") % (local_model.resource_id.replace("/", "_"),
                    EXTENSIONS[args.language]), "w") as handler:
        local_model.plug_in(out=handler)
    # creating a separate file to predict confidence
    if args.language in SEPARATE_OUTPUT:
        with open(os.path.join(args.output_dir, \
            "%s_confidence.%s") % (getattr(args, model_type).replace("/", "_"),
                        EXTENSIONS[args.language]), "w") as handler:
            local_model.plug_in(out=handler, attr=attr)


def export_code(args, api=None):
    """Generates the plugin code in the language required by the user

    """
    args.language = args.language or "javascript"

    if args.model is not None and args.language in EXPORTS:

        local_model = EXPORTS[args.language](args.model, api=api)
        generate_output(local_model, args, model_type="model")

    if args.ensemble is not None and args.language in EXPORTS:
        local_ensemble = Ensemble(args.ensemble, api=api)

        for model_id in local_ensemble.model_ids:
            local_model = EXPORTS[args.language]( \
                model_id,
                api=api,
                fields=local_ensemble.fields,
                boosting=local_ensemble.boosting)
            generate_output(local_model, args, model_type="model")


    """
    if args.logistic_regression is not None:
        if args.language not in LR_EXPORTS:
            sys.exit("Exporting to %s is not yet supported for this kind of "
                     "models." % args.language)
        local_logistic = LR_EXPORTS[args.language]( \
            args.logistic_regression, api=api)
        generate_output(local_logistic, args, model_type="logistic_regression",
                        attr="probability")
    """



# -*- coding: utf-8 -*-
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - Resources processing: creation, update and retrieval
of time-series

"""
from __future__ import absolute_import

import bigmler.utils as u
import bigmler.resources as r
import bigmler.checkpoint as c

from bigml.fields import Fields, DEFAULT_MISSING_TOKENS


def has_time_series(args):
    """Returns whether there's some kind of time-series ID

    """
    return args.time_series or args.time_series_set or \
        args.time_series_file


def time_series_processing(datasets, time_series, \
    time_series_ids, api, args, resume, fields=None, \
    session_file=None, path=None, log=None):
    """Creates or retrieves time_series from the input data

    """

    # If we have a dataset but not a model, we create the model if the no_model
    # flag hasn't been set up.
    if datasets and not (has_time_series(args) or \
            args.no_time_series):
        time_series_ids = []
        time_series_set = []

        # Only 1 time-series per bigmler command at present
        number_of_time_series = 1
        if resume:
            resume, time_series_ids = c.checkpoint( \
                c.are_time_series_created, path, \
                number_of_time_series, debug=args.debug)
            if not resume:
                message = u.dated("Found %s time-series out of %s."
                                  " Resuming.\n"
                                  % (len(time_series_ids),
                                     number_of_time_series))
                u.log_message(message, log_file=session_file,
                              console=args.verbosity)

            time_series_set = time_series_ids
            number_of_time_series -= len(time_series_ids)

        time_series_args = r.set_time_series_args( \
            args, fields=fields,
            objective_id=args.objective_id_)
        time_series_set, time_series_ids = \
            r.create_time_series( \
            datasets, time_series_set, time_series_args, \
            args, api, path, session_file, log)
    # If a time_series is provided, we use it.
    elif args.time_series:
        time_series_ids = [args.time_series]
        time_series_set = time_series_ids[:]

    elif args.time_series_set or args.time_series_tag:
        time_series_set = time_series_ids[:]

    # If we are going to predict we must retrieve the time-series
    if time_series_ids and args.export_fields:
        time_series_set, time_series_ids = \
            r.get_time_series(time_series_set, args, api, \
            session_file)

    return time_series_set, time_series_ids, resume



# -*- coding: utf-8 -*-
#
# Copyright 2014-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import absolute_import


import os
import time
try:
    import simplejson as json
except ImportError:
    import json
from bigmler.tests.world import world, res_filename
from subprocess import check_call
from bigml.api import check_resource

from bigmler.tests.common_steps import check_debug

from nose.tools import ok_, assert_equal


#@step(r'I create BigML resources using source to evaluate and log
# evaluation in "(.*)"')
def given_i_create_bigml_resources_using_source_to_evaluate(step,
                                                            output=None):
    if output is None:
        assert False
    command = ("bigmler --evaluate --source " + world.source['resource'] +
               " --output " + output)
    command = check_debug(command)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        world.directory = os.path.dirname(output)
        world.folders.append(world.directory)
        world.output = output
    except OSError as e:
        assert False


def read_id_from_file(path):
    """Reading the id in first line in a file

    """

    with open(path) as id_file:
        return id_file.readline().strip()


#@step(r'I create BigML resources using dataset to evaluate and log
# evaluation in "(.*)"')
def given_i_create_bigml_resources_using_dataset_to_evaluate(step,
                                                             output=None):
    if output is None:
        assert False
    command = ("bigmler --evaluate --dataset " + world.dataset['resource'] +
               " --output " + output)
    command = check_debug(command)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        world.directory = os.path.dirname(output)
        world.folders.append(world.directory)
        world.output = output
    except OSError as e:
        assert False


#@step(r'I create BigML resources using test file "([^"]*)" and a fields
# map "([^"]*)" to evaluate a model and log evaluation in "(.*)"')
def i_create_all_resources_to_evaluate_with_model_and_map( \
    step, data=None, fields_map=None, output=None):
    if data is None or fields_map is None or output is None:
        assert False
    command = ("bigmler --evaluate --test " + res_filename(data) +
               " --model " +
               world.model['resource'] + " --output " + output +
               " --fields-map " + res_filename(fields_map))
    command = check_debug(command)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        world.directory = os.path.dirname(output)
        world.folders.append(world.directory)
        world.output = output
    except OSError as e:
        assert False


#@step(r'I create BigML resources using test file "([^"]*)" to evaluate
# a model and log evaluation in "(.*)"')
def i_create_all_resources_to_evaluate_with_model(step,
                                                  data=None,
                                                  output=None):
    if data is None or output is None:
        assert False
    command = ("bigmler --evaluate --test " + res_filename(data) +
               " --model " +
               world.model['resource'] + " --output " + output)
    command = check_debug(command)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        world.directory = os.path.dirname(output)
        world.folders.append(world.directory)
        world.output = output
    except OSError as e:
        assert False


#@step(r'I create BigML resources using a dataset to evaluate a
# model and log evaluation in "(.*)"')
def given_i_create_bigml_resources_using_dataset_to_evaluate_with_model( \
    step, output=None):
    if output is None:
        assert False
    command = ("bigmler --evaluate --dataset " +
               world.dataset['resource']  + " --model " +
               world.model['resource'] + " --output " + output)
    command = check_debug(command)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        world.directory = os.path.dirname(output)
        world.folders.append(world.directory)
        world.output = output
    except OSError as e:
        assert False


#@step(r'I create BigML resources uploading train "(.*)" file to evaluate with
# test-split (.*) and log evaluation in "(.*)"')
def i_create_with_split_to_evaluate(step, data=None, split=None, output=None):
    if data is None or split is None or output is None:
        assert False
    command = ("bigmler --evaluate --train " + res_filename(data) +
               " --test-split " + split +  " --output " + output)
    command = check_debug(command)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        world.directory = os.path.dirname(output)
        world.folders.append(world.directory)
        world.output = output
    except OSError as e:
        assert False


#@step(r'I evaluate "(.*)" with proportional missing strategy')
def i_create_proportional_to_evaluate(step, test=None):
    if test is None:
        assert False
    test = res_filename(test)

    try:
        output_dir = world.directory + "_eval/"
        output = output_dir + os.path.basename(world.output)
        command = ("bigmler --evaluate --model " +
                   world.model['resource'] +
                   " --test " + test +
                   " --missing-strategy proportional --output " +
                   output)
        command = check_debug(command)
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        world.directory = output_dir
        world.folders.append(world.directory)
        world.output = output
    except OSError as e:
        assert False


#@step(r'the evaluation file is like "(.*)"$')
def then_the_evaluation_file_is_like(step, check_file_json):
    check_file_json = res_filename(check_file_json)
    evaluation_file_json = world.output + ".json"
    try:
        evaluation_file_json = open(evaluation_file_json, "U")
        check_file_json = open(check_file_json, "U")
        check = check_file_json.readline()
        evaluation = evaluation_file_json.readline()
        check = json.loads(check)
        evaluation = json.loads(evaluation)
        if 'model' in check:
            assert_equal(check['model'], evaluation['model'])
            assert_equal(check['mode'], evaluation['mode'])
        else:
            del check["datasets"]
            del evaluation["datasets"]
            assert_equal(check, evaluation)
        evaluation_file_json.close()
        check_file_json.close()
    except Exception, exc:
        assert False, str(exc)

#@step(r'I check that the (.*) dataset has been created$')
def i_check_create_dataset(step, dataset_type=None):
    dataset_file = "%s%sdataset_%s" % (world.directory, os.sep, dataset_type)
    try:
        dataset_file = open(dataset_file, "r")
        dataset = check_resource(dataset_file.readline().strip(),
                                 world.api.get_dataset)
        world.datasets.append(dataset['resource'])
        world.dataset = dataset
        dataset_file.close()
    except Exception, exc:
        assert False, str(exc)


#@step(r'the evaluation key "(.*)" value for the model is greater than (.*)$')
def i_check_evaluation_key(step, key=None, value=None):
    evaluation_file_json = world.output + ".json"
    try:
        evaluation_file_json = open(evaluation_file_json, "U")
        evaluation = evaluation_file_json.readline()
        evaluation = json.loads(evaluation)
        model_evaluation = evaluation['model']
        ok_(model_evaluation[key] > float(value),
            "model key: %s, expected >: %s" % ( \
            model_evaluation[key], float(value)))
        evaluation_file_json.close()
    except Exception, exc:
        assert False, str(exc)


#@step(r'I create BigML resources uploading train "(.*)" file to evaluate an
# ensemble of (\d+) models with test-split (.*) and log evaluation in "(.*)"')
def i_create_with_split_to_evaluate_ensemble( \
    step, data=None, number_of_models=None, split=None, output=None):
    if data is None or split is None or output is None:
        assert False
    command = ("bigmler --evaluate --train " + res_filename(data) +
               " --test-split " + split +
               " --number-of-models " + number_of_models +
               " --output " + output)
    command = check_debug(command)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        world.directory = os.path.dirname(output)
        world.folders.append(world.directory)
        world.output = output
    except OSError as e:
        assert False


#@step(r'I evaluate the ensemble in directory "(.*)" with the dataset in
# directory "(.*)" and log evaluation in "(.*)"')
def i_evaluate_ensemble_with_dataset( \
    step, ensemble_dir=None, dataset_dir=None, output=None):
    if ensemble_dir is None or dataset_dir is None or output is None:
        assert False
    world.directory = os.path.dirname(output)
    world.folders.append(world.directory)
    ensemble_id = read_id_from_file(os.path.join(ensemble_dir, "ensembles"))
    dataset_id = read_id_from_file(os.path.join(dataset_dir, "dataset_test"))
    command = ("bigmler --dataset " + dataset_id +
               " --ensemble " + ensemble_id + " --store" +
               " --output " + output + " --evaluate")
    command = check_debug(command)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        world.output = output
    except OSError as exc:
        assert False, str(exc)


#@step(r'I evaluate the ensemble in directory "(.*)" with the dataset in
# directory "(.*)" and log evaluation in "(.*)"')
def i_evaluate_ensemble_with_dataset_and_options( \
    step, ensemble_dir=None, dataset_dir=None, output=None, options=None):
    if ensemble_dir is None or dataset_dir is None or output is None \
            or options is None:
        assert False
    world.directory = os.path.dirname(output)
    world.folders.append(world.directory)
    ensemble_id = read_id_from_file(os.path.join(ensemble_dir, "ensembles"))
    dataset_id = read_id_from_file(os.path.join(dataset_dir, "dataset_test"))
    command = ("bigmler --dataset " + dataset_id +
               " --ensemble " + ensemble_id + " --store" +
               " --output " + output + " --evaluate " + options)
    command = check_debug(command)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        world.output = output
    except OSError as exc:
        assert False, str(exc)



# -*- coding: utf-8 -*-
#
# Copyright 2014-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - deepnet subcommand processing dispatching

"""
from __future__ import absolute_import

import sys
import os
import shutil

import bigml.api
import bigmler.utils as u
import bigmler.resources as r
import bigmler.pre_model_steps as pms
import bigmler.processing.args as a
import bigmler.processing.deepnets as pdn
import bigmler.processing.sources as ps
import bigmler.processing.datasets as pd

from bigmler.defaults import DEFAULTS_FILE
from bigmler.dnprediction import dn_prediction, remote_dn_prediction
from bigmler.reports import clear_reports, upload_reports
from bigmler.command import get_context
from bigmler.evaluation import evaluate
from bigmler.dispatcher import SESSIONS_LOG, clear_log_files, \
    get_test_dataset, get_objective_id

COMMAND_LOG = u".bigmler_deepnet"
DIRS_LOG = u".bigmler_deepnet_dir_stack"
LOG_FILES = [COMMAND_LOG, DIRS_LOG, u.NEW_DIRS_LOG]
MINIMUM_MODEL = "full=false"
DEFAULT_OUTPUT = u"predictions.csv"

SETTINGS = {
    "command_log": COMMAND_LOG,
    "sessions_log": SESSIONS_LOG,
    "dirs_log": DIRS_LOG,
    "default_output": DEFAULT_OUTPUT,
    "defaults_file": DEFAULTS_FILE}


def deepnet_dispatcher(args=sys.argv[1:]):
    """Parses command line and calls the different processing functions

    """

    # If --clear-logs the log files are cleared
    if "--clear-logs" in args:
        clear_log_files(LOG_FILES)

    settings = {}
    settings.update(SETTINGS)
    if '--evaluate' in args:
        settings.update({"default_output": "evaluation"})

    command_args, command, api, session_file, resume = get_context(args,
                                                                   settings)

    # Selects the action to perform
    if (a.has_train(command_args) or a.has_test(command_args)
            or command_args.export_fields):
        compute_output(api, command_args)
    u.log_message("_" * 80 + "\n", log_file=session_file)


def compute_output(api, args):
    """ Creates one or more models using the `training_set` or uses the ids
    of previously created BigML models to make predictions for the `test_set`.

    """

    deepnet = None
    deepnets = None

    # variables from command-line options
    resume = args.resume_
    deepnet_ids = args.deepnet_ids_
    output = args.predictions
    # there's only one deepnet to be generated at present
    args.max_parallel_deepnets = 1
    # deepnets cannot be published yet.
    args.public_deepnet = False

    # It is compulsory to have a description to publish either datasets or
    # deepnet
    if (not args.description_ and (args.public_deepnet or
                                   args.public_dataset)):
        sys.exit("You should provide a description to publish.")

    # When using --new-fields, it is compulsory to specify also a dataset
    # id
    if args.new_fields and not args.dataset:
        sys.exit("To use --new-fields you must also provide a dataset id"
                 " to generate the new dataset from it.")

    path = u.check_dir(output)
    session_file = u"%s%s%s" % (path, os.sep, SESSIONS_LOG)
    csv_properties = {}
    if args.objective_field:
        csv_properties.update({'objective_field': args.objective_field})
    # If logging is required set the file for logging
    log = None
    if args.log_file:
        u.check_dir(args.log_file)
        log = args.log_file
        # If --clear_logs the log files are cleared
        clear_log_files([log])

    # basic pre-model step: creating or retrieving the source related info
    source, resume, csv_properties, fields = pms.get_source_info(
        api, args, resume, csv_properties, session_file, path, log)
    # basic pre-model step: creating or retrieving the dataset related info
    dataset_properties = pms.get_dataset_info(
        api, args, resume, source,
        csv_properties, fields, session_file, path, log)
    (_, datasets, test_dataset,
     resume, csv_properties, fields) = dataset_properties
    if datasets:
        # Now we have a dataset, let's check if there's an objective_field
        # given by the user and update it in the fields structure
        args.objective_id_ = get_objective_id(args, fields)
    if args.deepnet_file:
        # deepnet is retrieved from the contents of the given local
        # JSON file
        deepnet, csv_properties, fields = u.read_local_resource(
            args.deepnet_file,
            csv_properties=csv_properties)
        deepnets = [deepnet]
        deepnet_ids = [deepnet['resource']]
    else:
        # deepnet is retrieved from the remote object
        deepnets, deepnet_ids, resume = \
            pdn.deepnets_processing( \
            datasets, deepnets, deepnet_ids, \
            api, args, resume, fields=fields, \
            session_file=session_file, path=path, log=log)
        if deepnets:
            deepnet = deepnets[0]

    # We update the deepnet's public state if needed
    if deepnet:
        if isinstance(deepnet, basestring) or \
                api.status(deepnet) != bigml.api.FINISHED:
            if not a.has_test(args):
                query_string = MINIMUM_MODEL
            elif args.export_fields:
                query_string = r.ALL_FIELDS_QS
            else:
                query_string = ''
            deepnet = u.check_resource(deepnet,
                                       api.get_deepnet,
                                       query_string=query_string)
        deepnets[0] = deepnet
        if (args.public_deepnet or
                (args.shared_flag and r.shared_changed(args.shared,
                                                       deepnet))):
            deepnet_args = {}
            if args.shared_flag and r.shared_changed(args.shared,
                                                     deepnet):
                deepnet_args.update(shared=args.shared)
            if args.public_deepnet:
                deepnet_args.update( \
                    r.set_publish_deepnet_args(args))
            if deepnet_args:
                deepnet = r.update_deepnet( \
                    deepnet, deepnet_args, args,
                    api=api, path=path, \
                    session_file=session_file)
                deepnet[0] = deepnet

    # We get the fields of the deepnet if we haven't got
    # them yet and need them
    if deepnet and (args.test_set or args.export_fields):
        fields = pdn.get_deepnet_fields( \
            deepnet, csv_properties, args)

    if fields and args.export_fields:
        fields.summary_csv(os.path.join(path, args.export_fields))

    # If predicting
    if deepnets and (a.has_test(args) or \
            (test_dataset and args.remote)):
        if test_dataset is None:
            test_dataset = get_test_dataset(args)

        # Remote predictions: predictions are computed as batch predictions
        # in bigml.com except when --no-batch flag is set on
        if args.remote and not args.no_batch:
            # create test source from file
            test_name = "%s - test" % args.name
            if args.test_source is None:
                test_properties = ps.test_source_processing(
                    api, args, resume, name=test_name,
                    session_file=session_file, path=path, log=log)
                (test_source, resume,
                 csv_properties, test_fields) = test_properties
            else:
                test_source_id = bigml.api.get_source_id(args.test_source)
                test_source = api.check_resource(test_source_id)
            if test_dataset is None:
                # create test dataset from test source
                dataset_args = r.set_basic_dataset_args(args, name=test_name)
                test_dataset, resume = pd.alternative_dataset_processing(
                    test_source, "test", dataset_args, api, args,
                    resume, session_file=session_file, path=path, log=log)
            else:
                test_dataset_id = bigml.api.get_dataset_id(test_dataset)
                test_dataset = api.check_resource(test_dataset_id)

            csv_properties.update(objective_field=None,
                                  objective_field_present=False)
            test_fields = pd.get_fields_structure(test_dataset,
                                                  csv_properties)
            batch_prediction_args = r.set_batch_prediction_args(
                args, fields=fields,
                dataset_fields=test_fields)

            remote_dn_prediction(deepnet, test_dataset, \
                batch_prediction_args, args, \
                api, resume, prediction_file=output, \
                session_file=session_file, path=path, log=log)

        else:
            dn_prediction(deepnets, fields, args,
                          session_file=session_file)

    # If evaluate flag is on, create remote evaluation and save results in
    # json and human-readable format.
    if args.evaluate:
        # When we resume evaluation and models were already completed, we
        # should use the datasets array as test datasets
        if args.has_test_datasets_:
            test_dataset = get_test_dataset(args)
        if args.dataset_off and not args.has_test_datasets_:
            args.test_dataset_ids = datasets
        if args.test_dataset_ids and args.dataset_off:
            # Evaluate the models with the corresponding test datasets.
            test_dataset_id = bigml.api.get_dataset_id( \
                args.test_dataset_ids[0])
            test_dataset = api.check_resource(test_dataset_id)
            csv_properties.update(objective_field=None,
                                  objective_field_present=False)
            test_fields = pd.get_fields_structure(test_dataset,
                                                  csv_properties)
            resume = evaluate(deepnets, args.test_dataset_ids, api,
                              args, resume,
                              fields=fields, dataset_fields=test_fields,
                              session_file=session_file, path=path,
                              log=log,
                              objective_field=args.objective_field)
        else:
            dataset = datasets[0]
            if args.test_split > 0 or args.has_test_datasets_:
                dataset = test_dataset
            dataset = u.check_resource(dataset, api=api,
                                       query_string=r.ALL_FIELDS_QS)
            dataset_fields = pd.get_fields_structure(dataset, None)
            resume = evaluate(deepnets, [dataset], api,
                              args, resume,
                              fields=fields, dataset_fields=dataset_fields,
                              session_file=session_file, path=path,
                              log=log,
                              objective_field=args.objective_field)


    u.print_generated_files(path, log_file=session_file,
                            verbosity=args.verbosity)
    if args.reports:
        clear_reports(path)
        if args.upload:
            upload_reports(args.reports, path)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Bigmler retrain

"""
from __future__ import absolute_import

import os

from bigml.api import BigML

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, teardown_class)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.project_creation_steps as test_project


try:
    BIGML_ORGANIZATION = os.environ['BIGML_ORGANIZATION']
except KeyError:
    raise ValueError("You need to set BIGML_ORGANIZATION"
                     " to an organization ID in your "
                     "environment variables to run this test.")


def setup_module():
    """Setup for the module

    """
    world.bck_api = world.api
    world.api = BigML(world.USERNAME, world.API_KEY, debug=world.debug,
                      organization=BIGML_ORGANIZATION)
    print world.api.connection_info()
    world.bck_project_id = world.project_id
    world.project_id = None
    world.clear()


def teardown_module():
    """Teardown for the module

    """

    if not world.debug:
        try:
            world.delete_resources()
        except Exception, exc:
            print exc
        world.api = BigML(world.USERNAME, world.API_KEY, debug=world.debug,
                          organization=BIGML_ORGANIZATION)
        project_stats = world.api.get_project( \
            world.project_id)['object']['stats']
        for resource_type, value in project_stats.items():
            if value['count'] != 0:
                # assert False, ("Increment in %s: %s" % (resource_type, value))
                print "WARNING: Increment in %s: %s" % (resource_type, value)

        world.api.delete_project(world.project_id)
    world.project_id = world.bck_project_id
    world.api = world.bck_api
    print world.api.connection_info()


class TestProjectOrg(object):

    def teardown(self):
        """Calling generic teardown for every method

        """
        print "\nEnd of tests in: %s\n-------------------\n" % __name__
        teardown_class()

    def setup(self):
        """
            Debug information
        """
        print "\n-------------------\nTests in: %s\n" % __name__

    def test_scenario1(self):
        """
            Scenario: Successfully creating a project and associating resources to it:
                Given I create a BigML project in an organization named "<project>" storing results in "<output_dir>"
                And I create a BigML source uploading train "<data>" file and associate it to the organization project storing results in "<output_dir>"
                And I check that the project has been created
                And I check that the source has been created
                Then the source is associated to the project

                Examples:
                | data             | project         | output_dir
                | ../data/iris.csv | My new project  | ./scenario_org_1
        """
        print self.test_scenario1.__doc__
        examples = [
            ['data/iris.csv', 'My new project', 'scenario_org_1']]
        for example in examples:
            print "\nTesting with:\n", example
            test_project.i_create_project_in_org(self, name=example[1], output_dir=example[2], organization=BIGML_ORGANIZATION)
            test_project.i_check_create_project(self, organization=True)
            test_project.i_create_source_with_org_project(self, data=example[0], output_dir=example[2])
            test_pred.i_check_create_source(self)
            test_project.check_source_in_project(self)



# -*- coding: utf-8 -*-
#
# Copyright 2017-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import absolute_import


import os
import time
import csv
import json

from nose.tools import assert_equal, assert_not_equal, ok_, eq_
from bigmler.tests.world import world, res_filename
from subprocess import check_call, CalledProcessError
from bigml.api import check_resource
from bigmler.checkpoint import file_number_of_lines
from bigmler.tests.common_steps import check_debug
from bigmler.export.dispatcher import EXTENSIONS
from bigml.util import PY3


def extract_content(file_handler):
    """reads the file and removes the model ID in the comments

    """
    content = file_handler.read()
    variable_comment = " from" \
        " model/"
    try:
        position = content.index(variable_comment)
        content = content[:position] + \
            content[position + len(variable_comment) + 24:]
    except ValueError:
        pass
    return content


def shell_execute(command, output, test=None, options=None,
                  data=None, test_split=None):
    """Execute bigmler command in shell

    """
    command = check_debug(command)
    world.directory = os.path.dirname(output)
    world.folders.append(world.directory)
    try:
        retcode = check_call(command, shell=True)
        if retcode < 0:
            assert False
        else:
            if test is not None:
                world.test_lines = file_number_of_lines(test)
                world.test_lines -= 1
                # prediction file has headers in it,
                # so first line must be ignored
                world.prediction_header = options is not None and \
                    options.find('--prediction-header') > -1
            if test_split is not None:
                data_lines = file_number_of_lines(data) - 1
                world.test_lines = int(data_lines * float(test_split))
            world.output = output
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)


#@step(r'I I export the model as a function in "(.*)"$')
def i_export_model(step, language=None, output=None):
    ok_(language is not None and output is not None)
    output_dir = world.directory
    command = ("bigmler export --language " + language +
               " --output-dir " + output_dir + " --model " +
               world.model['resource'])
    shell_execute(command, output)

#@step(r'I create BigML resources uploading train "(.*)" file using "(.*)" and log in "(.*)"$')
def i_create_all_resources_to_model_with_source_attrs( \
    self, data=None, source_attributes=None, output=None):
    ok_(data is not None and source_attributes is not None
        and output is not None)
    if source_attributes != "":
        source_attributes = " --source-attributes " + \
            res_filename(source_attributes)
    command = ("bigmler --train " + res_filename(data) +
               " --output " + output + source_attributes +
               " --store --max-batch-models 1 --no-fast")
    shell_execute(command, output)

#@step(r'I check the output for "(.*)" is like "(.*)" expected file')
def i_check_if_the_output_is_like_expected_file(step, language, expected_file):
    ok_(language is not None and expected_file is not None)
    with open(res_filename(expected_file)) as file_handler:
        expected_content = extract_content(file_handler)
    with open(os.path.join(world.directory, "%s.%s" % \
            (world.model['resource'].replace("/", "_"),
             EXTENSIONS[language]))) \
            as file_handler:
        output = extract_content(file_handler)
    if PY3 and output.strip() != expected_content.strip():
        try:
            name, extension = expected_file.split(".")
            expected_file = "%s_p3.%s" % (name, extension)
            with open(res_filename(expected_file)) as file_handler:
                expected_content = extract_content(file_handler)
        except IOError:
            pass
    eq_(output.strip(), expected_content.strip(), "Found: %s\nExpected:%s" % \
        (world.directory + "/" + world.model["resource"],
         expected_file))



# -*- coding: utf-8 -*-
#
# Copyright 2015-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - project creation and update

"""
from __future__ import absolute_import

import sys
import os
import shutil

import bigmler.utils as u
import bigmler.processing.args as a
import bigmler.processing.projects as pp

from bigmler.defaults import DEFAULTS_FILE
from bigmler.command import get_context
from bigmler.dispatcher import SESSIONS_LOG, clear_log_files

COMMAND_LOG = u".bigmler_project"
DIRS_LOG = u".bigmler_project_dir_stack"
LOG_FILES = [COMMAND_LOG, DIRS_LOG, u.NEW_DIRS_LOG]

SETTINGS = {
    "command_log": COMMAND_LOG,
    "sessions_log": SESSIONS_LOG,
    "dirs_log": DIRS_LOG,
    "defaults_file": DEFAULTS_FILE}


def project_dispatcher(args=sys.argv[1:]):
    """Parses command line and calls the different processing functions

    """

    command_args, command, api, session_file, resume = get_context(args,
                                                                   SETTINGS)

    path = u.check_dir(command_args.output)
    log = None
    if command_args.log_file:
        u.check_dir(command_args.log_file)
        log = command_args.log_file
        # If --clear_logs the log files are cleared
        clear_log_files([log])
    if not command_args.project_id and command_args.name:
        command_args.project = command_args.name
    if command_args.project:
        # create project
        pp.project_processing(
            api, command_args, command_args.resume, session_file=session_file,
            path=path, log=log, create=True)
    if command_args.project_id and (
            command_args.project_attributes or
            command_args.name or command_args.tag or command_args.description
            or command_args.category):
        # update project's attributes
        pp.update_project(command_args, api, command_args.resume, \
            session_file=session_file)

    u.log_message("_" * 80 + "\n", log_file=session_file)
    u.print_generated_files(command_args.output_dir, log_file=session_file,
                            verbosity=command_args.verbosity)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2015-2018 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing associations

"""
from __future__ import absolute_import


from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, teardown_class)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_association_steps as test_association


def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestAssociation(object):

    def teardown(self):
        """Calling generic teardown for every method

        """
        print "\nEnd of tests in: %s\n-------------------\n" % __name__
        teardown_class()

    def setup(self):
        """
            Debug information
        """
        print "\n-------------------\nTests in: %s\n" % __name__

    def test_scenario1(self):
        """
            Scenario: Successfully building association from scratch:
                Given I create BigML association uploading train "<data>" file and log resources in "<output_dir>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the association has been created

                Examples:
                | data               | output_dir
                | ../data/grades.csv |  ./scenario_ass_1_r
                | ../data/diabetes.csv   | ./scenario_ass_1
        """
        print self.test_scenario1.__doc__
        examples = [
            ['data/spam.csv', 'scenario_ass_1_r'],
            ['data/movies.csv', 'scenario_ass_1_i'],
            ['data/iris.csv', 'scenario_ass_1']]
        for example in examples:
            print "\nTesting with:\n", example
            test_association.i_create_association(self, data=example[0], output_dir=example[1])
            test_pred.i_check_create_source(self)
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_association(self)

    def test_scenario2(self):
        """
            Scenario: Successfully building association from source
                Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
                And I create BigML association using source and log resources in "<output_dir>"
                And I check that the dataset has been created
                And I check that the association has been created

                Examples:
                |scenario    | kwargs                                                  | output_dir
                | scenario_ass_1| {"data": "../data/iris.csv", "output_dir": "./scenario_ass_1/}   | ./scenario_ass_2   |
        """
        print self.test_scenario2.__doc__
        examples = [
            ['scenario_ass_1', '{"data": "data/iris.csv", "output_dir": "scenario_ass_1"}', 'scenario_ass_2']]
        for example in examples:
            print "\nTesting with:\n", example
            test_pred.i_have_previous_scenario_or_reproduce_it(self, example[0], example[1])
            test_association.i_create_association_from_source(self, output_dir=example[2])
            test_pred.i_check_create_dataset(self, suffix=None)
            test_pred.i_check_create_association(self)

    def test_scenario3(self):
        """
            Scenario: Successfully building test predictions from dataset
                Given I have previously executed "<scenario>" or reproduce it with arguments <kwargs>
                And I create BigML association using dataset and log predictions in "<output_dir>"
                And I check that the association has been created

                Examples:
                |scenario    | kwargs                                                  | test                    | output                        |predictions_file           |
                | scenario_ass_1| {"data": "../data/iris.csv", "output_dir": "./scenario_c_1"}   | ../data/diabetes.csv   | ./scenario_c_3/centroids.csv   | ./check_files/centroids_diabetes.csv   |

        """
        print self.test_scenario3.__doc__
        examples = [
            ['scenario_ass_1', '{"data": "data/iris.csv", "output_dir": "scenario_ass_1"}', 'scenario_ass_3']]
        for example in examples:
            print "\nTesting with:\n", example
            test_pred.i_have_previous_scenario_or_reproduce_it(self, example[0], example[1])
            test_association.i_create_association_from_dataset(self, output_dir=example[2])
            test_pred.i_check_create_association(self)
